@@ -37,10 +37,6 @@
 #include "iscsi.h"
 #include "scsi-lowlevel.h"
 
-#ifndef discard_const
-#define discard_const(ptr) ((void *)((intptr_t)(ptr)))
-#endif
-
 int showluns;
 const char *initiator = "iqn.2007-10.com.github:sahlberg:libiscsi:iscsi-ls";
 
@@ -223,7 +219,7 @@ void list_luns(struct client_state *clnt, const char *target, const char *portal
 
 
 
-void discoverylogout_cb(struct iscsi_context *iscsi, int status, void *command_data _U_, void *private_data)
+void discoverylogout_cb(struct iscsi_context *iscsi, int status, void *command_data, void *private_data)
 {
 	struct client_state *state = (struct client_state *)private_data;
 
@@ -263,7 +259,7 @@ void discovery_cb(struct iscsi_context *iscsi, int status, void *command_data, v
 }
 
 
-void discoverylogin_cb(struct iscsi_context *iscsi, int status, void *command_data _U_, void *private_data)
+void discoverylogin_cb(struct iscsi_context *iscsi, int status, void *command_data, void *private_data)
 {
 	if (status != 0) {
 		fprintf(stderr, "Login failed. %s\n", iscsi_get_error(iscsi));
@@ -276,7 +272,7 @@ void discoverylogin_cb(struct iscsi_context *iscsi, int status, void *command_da
 	}
 }
 
-void discoveryconnect_cb(struct iscsi_context *iscsi, int status, void *command_data _U_, void *private_data)
+void discoveryconnect_cb(struct iscsi_context *iscsi, int status, void *command_data, void *private_data)
 {
 	if (status != 0) {
 		fprintf(stderr, "discoveryconnect_cb: connection failed : %s\n", iscsi_get_error(iscsi));
@@ -318,7 +314,7 @@ int main(int argc, char *argv[])
 	struct iscsi_context *iscsi;
 	struct iscsi_url *iscsi_url = NULL;
 	struct client_state state;
-	const char *url = NULL;
+	char *url = NULL;
 	int c;
 	static int show_help = 0, show_usage = 0, debug = 0;
 
@@ -384,9 +380,7 @@ int main(int argc, char *argv[])
 
 	iscsi_url = iscsi_parse_portal_url(iscsi, url);
 
-	if (url) {
-		free(discard_const(url));
-	}
+        free(url);
 
 	if (iscsi_url == NULL) {
 		fprintf(stderr, "Failed to parse URL: %s\n",
