@@ -15,15 +15,21 @@
    along with this program; if not, see <http://www.gnu.org/licenses/>.
 */
 
+#include "config.h"
 #include <stdio.h>
 #include <stdlib.h>
 #include <stdint.h>
 #include <inttypes.h>
 #include <string.h>
+#ifdef HAVE_POLL_H
 #include <poll.h>
+#else
+#include "win32_compat.h"
+#endif
 #include <getopt.h>
 #include <unistd.h>
 #include <limits.h>
+#include <time.h>
 #include "iscsi.h"
 #include "scsi-lowlevel.h"
 
@@ -31,22 +37,23 @@ const char *initiator = "iqn.2010-11.ronnie:iscsi-inq";
 uint32_t max_in_flight = 50;
 uint32_t blocks_per_io = 200;
 
+struct iscsi_endpoint {
+	struct iscsi_context *iscsi;
+	int lun;
+	int blocksize;
+	uint64_t num_blocks;
+	struct scsi_inquiry_device_designator tgt_desig;
+};
+
 struct client {
 	int finished;
 	uint32_t in_flight;
 
-	struct iscsi_context *src_iscsi;
-	int src_lun;
-	int src_blocksize;
-	uint64_t src_num_blocks;
-	struct scsi_inquiry_device_designator src_tgt_desig;
+	struct iscsi_endpoint src;
+	struct iscsi_endpoint dst;
+
 	uint64_t pos;
 
-	struct iscsi_context *dst_iscsi;
-	int dst_lun;
-	int dst_blocksize;
-	uint64_t dst_num_blocks;
-	struct scsi_inquiry_device_designator dst_tgt_desig;
 	int use_16_for_rw;
 	int use_xcopy;
 	int progress;
@@ -69,13 +76,13 @@ void write_cb(struct iscsi_context *iscsi, int status, void *command_data, void
 	struct client *client = wt->client;
 
 	if (status == SCSI_STATUS_CHECK_CONDITION) {
-		printf("Write10/16 failed with sense key:%d ascq:%04x\n", task->sense.key, task->sense.ascq);
+		fprintf(stderr, "Write10/16 failed with sense key:%d ascq:%04x\n", task->sense.key, task->sense.ascq);
 		scsi_free_scsi_task(task);
 		exit(10);
 	}
 
 	if (status != SCSI_STATUS_GOOD) {
-		printf("Write10/16 failed with %s\n", iscsi_get_error(iscsi));
+		fprintf(stderr, "Write10/16 failed with %s\n", iscsi_get_error(iscsi));
 		if (!client->ignore_errors) {
 			scsi_free_scsi_task(task);
 			exit(10);
@@ -86,10 +93,10 @@ void write_cb(struct iscsi_context *iscsi, int status, void *command_data, void
 	fill_read_queue(client);
 
 	if (client->progress) {
-		printf("\r%"PRIu64" of %"PRIu64" blocks transferred.", client->pos, client->src_num_blocks);
+		printf("\r%"PRIu64" of %"PRIu64" blocks transferred.", client->pos, client->src.num_blocks);
 	}
 
-	if ((client->in_flight == 0) && (client->pos == client->src_num_blocks)) {
+	if ((client->in_flight == 0) && (client->pos == client->src.num_blocks)) {
 		client->finished = 1;
 		if (client->progress) {
 			printf("\n");
@@ -100,7 +107,7 @@ void write_cb(struct iscsi_context *iscsi, int status, void *command_data, void
 	free(wt);
 }
 
-void read_cb(struct iscsi_context *iscsi _U_, int status, void *command_data, void *private_data)
+void read_cb(struct iscsi_context *iscsi, int status, void *command_data, void *private_data)
 {
 	struct client *client = (struct client *)private_data;
 	struct scsi_task *task = command_data;
@@ -110,13 +117,13 @@ void read_cb(struct iscsi_context *iscsi _U_, int status, void *command_data, vo
 	struct scsi_task *task2;
 
 	if (status == SCSI_STATUS_CHECK_CONDITION) {
-		printf("Read10/16 failed with sense key:%d ascq:%04x\n", task->sense.key, task->sense.ascq);
+		fprintf(stderr, "Read10/16 failed with sense key:%d ascq:%04x\n", task->sense.key, task->sense.ascq);
 		scsi_free_scsi_task(task);
 		exit(10);
 	}
 
 	if (status != SCSI_STATUS_GOOD) {
-		printf("Read10/16 failed with %s\n", iscsi_get_error(iscsi));
+		fprintf(stderr, "Read10/16 failed with %s\n", iscsi_get_error(iscsi));
 		if (!client->ignore_errors) {
 			scsi_free_scsi_task(task);
 			exit(10);
@@ -124,32 +131,37 @@ void read_cb(struct iscsi_context *iscsi _U_, int status, void *command_data, vo
 	}
 
 	wt = malloc(sizeof(struct write_task));
+	if (wt == NULL) {
+		fprintf(stderr, "failed to alloc write task\n");
+		exit(10);
+	}
+	
 	wt->rt = task;
 	wt->client = client;
 
 	if (client->use_16_for_rw) {
 		read16_cdb = scsi_cdb_unmarshall(task, SCSI_OPCODE_READ16);
 		if (read16_cdb == NULL) {
-			printf("Failed to unmarshall READ16 CDB.\n");
+			fprintf(stderr, "Failed to unmarshall READ16 CDB.\n");
 			exit(10);
 		}
-		task2 = iscsi_write16_task(client->dst_iscsi, client->dst_lun,
-									read16_cdb->lba, task->datain.data, task->datain.size,
-									client->dst_blocksize, 0, 0, 0, 0, 0,
-									write_cb, wt);
+		task2 = iscsi_write16_task(client->dst.iscsi, client->dst.lun,
+					   read16_cdb->lba, task->datain.data, task->datain.size,
+					   client->dst.blocksize, 0, 0, 0, 0, 0,
+					   write_cb, wt);
 	} else {
 		read10_cdb = scsi_cdb_unmarshall(task, SCSI_OPCODE_READ10);
 		if (read10_cdb == NULL) {
-			printf("Failed to unmarshall READ16 CDB.\n");
+			fprintf(stderr, "Failed to unmarshall READ10 CDB.\n");
 			exit(10);
 		}
-		task2 = iscsi_write10_task(client->dst_iscsi, client->dst_lun,
-									read10_cdb->lba, task->datain.data, task->datain.size,
-									client->dst_blocksize, 0, 0, 0, 0, 0,
-									write_cb, wt);
+		task2 = iscsi_write10_task(client->dst.iscsi, client->dst.lun,
+					   read10_cdb->lba, task->datain.data, task->datain.size,
+					   client->dst.blocksize, 0, 0, 0, 0, 0,
+					   write_cb, wt);
 	}
 	if (task2 == NULL) {
-		printf("failed to send read16 command\n");
+		fprintf(stderr, "failed to send write10/16 command\n");
 		scsi_free_scsi_task(task);
 		exit(10);
 	}
@@ -160,30 +172,30 @@ void fill_read_queue(struct client *client)
 {
 	uint32_t num_blocks;
 
-	while(client->in_flight < max_in_flight && client->pos < client->src_num_blocks) {
+	while(client->in_flight < max_in_flight && client->pos < client->src.num_blocks) {
 		struct scsi_task *task;
 		client->in_flight++;
 
-		num_blocks = client->src_num_blocks - client->pos;
+		num_blocks = client->src.num_blocks - client->pos;
 		if (num_blocks > blocks_per_io) {
 			num_blocks = blocks_per_io;
 		}
 
 		if (client->use_16_for_rw) {
-			task = iscsi_read16_task(client->src_iscsi,
-									client->src_lun, client->pos,
-									num_blocks * client->src_blocksize,
-									client->src_blocksize, 0, 0, 0, 0, 0,
-									read_cb, client);
+			task = iscsi_read16_task(client->src.iscsi,
+						 client->src.lun, client->pos,
+						 num_blocks * client->src.blocksize,
+						 client->src.blocksize, 0, 0, 0, 0, 0,
+						 read_cb, client);
 		} else {
-			task = iscsi_read10_task(client->src_iscsi,
-									client->src_lun, client->pos,
-									num_blocks * client->src_blocksize,
-									client->src_blocksize, 0, 0, 0, 0, 0,
-									read_cb, client);
+			task = iscsi_read10_task(client->src.iscsi,
+						 client->src.lun, client->pos,
+						 num_blocks * client->src.blocksize,
+						 client->src.blocksize, 0, 0, 0, 0, 0,
+						 read_cb, client);
 		}
 		if (task == NULL) {
-			printf("failed to send read10/16 command\n");
+			fprintf(stderr, "failed to send read10/16 command\n");
 			exit(10);
 		}
 		client->pos += num_blocks;
@@ -274,20 +286,20 @@ void populate_param_header(unsigned char *buf, int list_id, int str, int list_id
 	buf[15] = inline_data_len & 0xFF;
 }
 
-void xcopy_cb(struct iscsi_context *iscsi _U_, int status, void *command_data, void *private_data)
+void xcopy_cb(struct iscsi_context *iscsi, int status, void *command_data, void *private_data)
 {
 	struct client *client = (struct client *)private_data;
 	struct scsi_task *task = command_data;
 
 	if (status == SCSI_STATUS_CHECK_CONDITION) {
-		printf("XCOPY failed with sense key:%d ascq:%04x\n",
+		fprintf(stderr, "XCOPY failed with sense key:%d ascq:%04x\n",
 			task->sense.key, task->sense.ascq);
 		scsi_free_scsi_task(task);
 		exit(10);
 	}
 
 	if (status != SCSI_STATUS_GOOD) {
-		printf("XCOPY failed with %s\n", iscsi_get_error(iscsi));
+		fprintf(stderr, "XCOPY failed with %s\n", iscsi_get_error(iscsi));
 		if (!client->ignore_errors) {
 			scsi_free_scsi_task(task);
 			exit(10);
@@ -299,10 +311,10 @@ void xcopy_cb(struct iscsi_context *iscsi _U_, int status, void *command_data, v
 
 	if (client->progress) {
 		printf("\r%"PRIu64" of %"PRIu64" blocks transferred.",
-			client->pos, client->src_num_blocks);
+			client->pos, client->src.num_blocks);
 	}
 
-	if ((client->in_flight == 0) && (client->pos == client->src_num_blocks)) {
+	if ((client->in_flight == 0) && (client->pos == client->src.num_blocks)) {
 		client->finished = 1;
 		if (client->progress) {
 			printf("\n");
@@ -313,7 +325,7 @@ void xcopy_cb(struct iscsi_context *iscsi _U_, int status, void *command_data, v
 
 void fill_xcopy_queue(struct client *client)
 {
-	while (client->in_flight < max_in_flight && client->pos < client->src_num_blocks) {
+	while (client->in_flight < max_in_flight && client->pos < client->src.num_blocks) {
 		struct scsi_task *task;
 		struct iscsi_data data;
 		unsigned char *xcopybuf;
@@ -324,7 +336,7 @@ void fill_xcopy_queue(struct client *client)
 
 		client->in_flight++;
 
-		num_blocks = client->src_num_blocks - client->pos;
+		num_blocks = client->src.num_blocks - client->pos;
 		if (num_blocks > blocks_per_io) {
 			num_blocks = blocks_per_io;
 		}
@@ -334,7 +346,7 @@ void fill_xcopy_queue(struct client *client)
 			28;		/* BLK_TO_BLK_SEG_DESCR */
 		data.data = malloc(data.size);
 		if (data.data == NULL) {
-			printf("failed to alloc XCOPY buffer\n");
+			fprintf(stderr, "failed to alloc XCOPY buffer\n");
 			exit(10);
 		}
 
@@ -344,11 +356,11 @@ void fill_xcopy_queue(struct client *client)
 		/* Initialise CSCD list with one src + one dst descriptor */
 		offset = XCOPY_DESC_OFFSET;
 		offset += populate_tgt_desc(xcopybuf + offset,
-					&client->src_tgt_desig,
-					0, client->src_blocksize);
+					&client->src.tgt_desig,
+					0, client->src.blocksize);
 		offset += populate_tgt_desc(xcopybuf + offset,
-					&client->dst_tgt_desig,
-					0, client->dst_blocksize);
+					&client->dst.tgt_desig,
+					0, client->dst.blocksize);
 		tgt_desc_len = offset - XCOPY_DESC_OFFSET;
 
 		/* Initialise one segment descriptor */
@@ -360,11 +372,11 @@ void fill_xcopy_queue(struct client *client)
 		populate_param_header(xcopybuf, 1, 0, LIST_ID_USAGE_DISCARD, 0,
 				tgt_desc_len, seg_desc_len, 0);
 
-		task = iscsi_extended_copy_task(client->src_iscsi,
-						client->src_lun,
+		task = iscsi_extended_copy_task(client->src.iscsi,
+						client->src.lun,
 						&data, xcopy_cb, client);
 		if (task == NULL) {
-			printf("failed to send XCOPY command\n");
+			fprintf(stderr, "failed to send XCOPY command\n");
 			exit(10);
 		}
 
@@ -413,7 +425,7 @@ void cscd_ident_inq(struct iscsi_context *iscsi,
 	}
 
 	if (tgt_desig == NULL) {
-		fprintf(stderr, "No suitalble target descriptor format found");
+		fprintf(stderr, "No suitable target descriptor format found\n");
 		exit(10);
 	}
 
@@ -423,6 +435,11 @@ void cscd_ident_inq(struct iscsi_context *iscsi,
 	_tgt_desig->designator_type = tgt_desig->designator_type;
 	_tgt_desig->designator_length = tgt_desig->designator_length;
 	_tgt_desig->designator = malloc(tgt_desig->designator_length);
+	if (_tgt_desig->designator == NULL) {
+		fprintf(stderr, "failed to alloc designator\n");
+		exit(10);
+	}
+	
 	memcpy(_tgt_desig->designator, tgt_desig->designator, tgt_desig->designator_length);
 
 	scsi_free_scsi_task(task);
@@ -523,15 +540,102 @@ void readcap(struct iscsi_context *iscsi, int lun, int use_16,
 	return;
 }
 
+static void usage_exit(int status)
+{
+	fprintf(stderr, "Usage:\n"
+"-s, --src <URL>               source iSCSI URL             (required)\n"
+"-d, --dst <URL>               destination iSCSI URL        (required)\n"
+"-i, --initiator-name <IQN>    iSCSI initiator name         (default=%s)\n"
+"-p, --progress                show progress while copying\n"
+"-6, --16                      use READ16 & WRITE16 SCSI commands\n"
+"-x, --xcopy                   offload I/O to the target via XCOPY\n"
+"-m, --max <NUM>               maximum requests in flight   (default=%u)\n"
+"-b, --blocks <NUM>            blocks per I/O               (default=%u)\n"
+"-n, --ignore-errors           ignore any I/O errors\n"
+"-h, --help                    show this usage message\n",
+		initiator, max_in_flight, blocks_per_io);
+	exit(status);
+}
+
+static void show_perf(struct timespec *start_time,
+		      struct timespec *end_time,
+		      uint64_t num_blocks,
+		      uint64_t block_size)
+{
+	const char u[] = { 'b', 'K', 'M', 'G', 'T'};
+	double elapsed = (end_time->tv_sec + 1.0e-9 * end_time->tv_nsec)
+			- (start_time->tv_sec + 1.0e-9 * start_time->tv_nsec);
+	double ubytes_per_sec = num_blocks * block_size / elapsed;
+	unsigned int i = 0;
+
+	while (ubytes_per_sec > 1024 && i < sizeof(u) - 1) {
+		ubytes_per_sec = ubytes_per_sec / 1024;
+		i++;
+	}
+
+	printf("\r%"PRIu64" blocks (%"PRIu64" sized) copied in %g seconds,"
+	   " %g%c/s.\n", num_blocks, block_size, elapsed, ubytes_per_sec, u[i]);
+}
+
+static void iscsi_endpoint_init(const char *url,
+				const char *usage,
+				int use_16_for_rw,
+				int use_xcopy,
+				struct iscsi_endpoint *endpoint)
+{
+	struct iscsi_url *iscsi_url;
+
+	if (url == NULL) {
+		fprintf(stderr, "You must specify a %s url\n"
+			"  --%s iscsi://<host>[:<port>]/<target-iqn>/<lun>\n",
+			usage, usage);
+		usage_exit(10);
+	}
+
+	endpoint->iscsi = iscsi_create_context(initiator);
+	if (endpoint->iscsi == NULL) {
+		fprintf(stderr, "Failed to create context\n");
+		exit(10);
+	}
+	iscsi_url = iscsi_parse_full_url(endpoint->iscsi, url);
+	if (iscsi_url == NULL) {
+		fprintf(stderr, "Failed to parse URL: %s\n",
+			iscsi_get_error(endpoint->iscsi));
+		iscsi_destroy_context(endpoint->iscsi);
+		exit(10);
+	}
+	iscsi_set_session_type(endpoint->iscsi, ISCSI_SESSION_NORMAL);
+	iscsi_set_header_digest(endpoint->iscsi, ISCSI_HEADER_DIGEST_NONE_CRC32C);
+	if (iscsi_full_connect_sync(endpoint->iscsi, iscsi_url->portal, iscsi_url->lun) != 0) {
+		fprintf(stderr, "Login Failed. %s\n", iscsi_get_error(endpoint->iscsi));
+		iscsi_destroy_url(iscsi_url);
+		iscsi_destroy_context(endpoint->iscsi);
+		exit(10);
+	}
+	endpoint->lun = iscsi_url->lun;
+	iscsi_destroy_url(iscsi_url);
+
+	readcap(endpoint->iscsi, endpoint->lun, use_16_for_rw,
+		&endpoint->blocksize, &endpoint->num_blocks);
+
+	if (use_xcopy) {
+		cscd_ident_inq(endpoint->iscsi, endpoint->lun,
+				&endpoint->tgt_desig);
+		cscd_param_check(endpoint->iscsi, endpoint->lun,
+				 endpoint->blocksize);
+	}
+}
+
 int main(int argc, char *argv[])
 {
 	char *src_url = NULL;
 	char *dst_url = NULL;
-	struct iscsi_url *iscsi_url;
 	int c;
 	struct pollfd pfd[2];
 	struct client client;
-
+	struct timespec start_time;
+	struct timespec end_time;
+	int gettime_ret;
 	static struct option long_options[] = {
 		{"dst",            required_argument,    NULL,        'd'},
 		{"src",            required_argument,    NULL,        's'},
@@ -542,13 +646,14 @@ int main(int argc, char *argv[])
 		{"max",            required_argument,    NULL,        'm'},
 		{"blocks",         required_argument,    NULL,        'b'},
 		{"ignore-errors",  no_argument,          NULL,        'n'},
+		{"help",           no_argument,          NULL,        'h'},
 		{0, 0, 0, 0}
 	};
 	int option_index;
 
 	memset(&client, 0, sizeof(client));
 
-	while ((c = getopt_long(argc, argv, "d:s:i:m:b:p6nx", long_options,
+	while ((c = getopt_long(argc, argv, "d:s:i:m:b:p6nxh", long_options,
 			&option_index)) != -1) {
 		char *endptr;
 
@@ -590,95 +695,33 @@ int main(int argc, char *argv[])
 		case 'n':
 			client.ignore_errors = 1;
 			break;
+		case 'h':
+			usage_exit(0);
+			break;
 		default:
 			fprintf(stderr, "Unrecognized option '%c'\n\n", c);
-			exit(1);
+			usage_exit(1);
 		}
 	}
 
-	if (src_url == NULL) {
-		fprintf(stderr, "You must specify source url\n");
-		fprintf(stderr, "  --src iscsi://<host>[:<port>]/<target-iqn>/<lun>\n");
-		exit(10);
-	}
-	if (dst_url == NULL) {
-		fprintf(stderr, "You must specify destination url\n");
-		fprintf(stderr, "  --dst iscsi://<host>[:<port>]/<target-iqn>/<lun>\n");
+	iscsi_endpoint_init(src_url, "src", client.use_16_for_rw,
+			    client.use_xcopy, &client.src);
+	iscsi_endpoint_init(dst_url, "dst", client.use_16_for_rw,
+			    client.use_xcopy, &client.dst);
+
+	if (client.src.blocksize != client.dst.blocksize) {
+		fprintf(stderr, "source LUN has different blocksize than destination (%d != %d)\n", client.src.blocksize, client.dst.blocksize);
 		exit(10);
 	}
 
-	client.src_iscsi = iscsi_create_context(initiator);
-	if (client.src_iscsi == NULL) {
-		fprintf(stderr, "Failed to create context\n");
-		exit(10);
-	}
-	iscsi_url = iscsi_parse_full_url(client.src_iscsi, src_url);
-	if (iscsi_url == NULL) {
-		fprintf(stderr, "Failed to parse URL: %s\n",
-			iscsi_get_error(client.src_iscsi));
-		exit(10);
-	}
-	iscsi_set_session_type(client.src_iscsi, ISCSI_SESSION_NORMAL);
-	iscsi_set_header_digest(client.src_iscsi, ISCSI_HEADER_DIGEST_NONE_CRC32C);
-	if (iscsi_full_connect_sync(client.src_iscsi, iscsi_url->portal, iscsi_url->lun) != 0) {
-		fprintf(stderr, "Login Failed. %s\n", iscsi_get_error(client.src_iscsi));
-		iscsi_destroy_url(iscsi_url);
-		iscsi_destroy_context(client.src_iscsi);
-		exit(10);
-	}
-	client.src_lun = iscsi_url->lun;
-	iscsi_destroy_url(iscsi_url);
-
-	readcap(client.src_iscsi, client.src_lun, client.use_16_for_rw,
-		&client.src_blocksize, &client.src_num_blocks);
-
-	if (client.use_xcopy) {
-		cscd_ident_inq(client.src_iscsi, client.src_lun,
-				&client.src_tgt_desig);
-		cscd_param_check(client.src_iscsi, client.src_lun,
-				 client.src_blocksize);
-	}
-
-	client.dst_iscsi = iscsi_create_context(initiator);
-	if (client.dst_iscsi == NULL) {
-		fprintf(stderr, "Failed to create context\n");
-		exit(10);
-	}
-	iscsi_url = iscsi_parse_full_url(client.dst_iscsi, dst_url);
-	if (iscsi_url == NULL) {
-		fprintf(stderr, "Failed to parse URL: %s\n",
-			iscsi_get_error(client.dst_iscsi));
-		exit(10);
-	}
-	iscsi_set_session_type(client.dst_iscsi, ISCSI_SESSION_NORMAL);
-	iscsi_set_header_digest(client.dst_iscsi, ISCSI_HEADER_DIGEST_NONE_CRC32C);
-	if (iscsi_full_connect_sync(client.dst_iscsi, iscsi_url->portal, iscsi_url->lun) != 0) {
-		fprintf(stderr, "Login Failed. %s\n", iscsi_get_error(client.dst_iscsi));
-		iscsi_destroy_url(iscsi_url);
-		iscsi_destroy_context(client.dst_iscsi);
-		exit(10);
-	}
-	client.dst_lun = iscsi_url->lun;
-	iscsi_destroy_url(iscsi_url);
-
-	readcap(client.dst_iscsi, client.dst_lun, client.use_16_for_rw,
-		&client.dst_blocksize, &client.dst_num_blocks);
-
-	if (client.use_xcopy) {
-		cscd_ident_inq(client.dst_iscsi, client.dst_lun,
-				&client.dst_tgt_desig);
-		cscd_param_check(client.dst_iscsi, client.dst_lun,
-				 client.dst_blocksize);
-	}
-
-	if (client.src_blocksize != client.dst_blocksize) {
-		fprintf(stderr, "source LUN has different blocksize than destination than destination (%d != %d sectors)\n", client.src_blocksize, client.dst_blocksize);
+	if (client.src.num_blocks > client.dst.num_blocks) {
+		fprintf(stderr, "source LUN is bigger than destination (%"PRIu64" > %"PRIu64" sectors)\n", client.src.num_blocks, client.dst.num_blocks);
 		exit(10);
 	}
 
-	if (client.src_num_blocks > client.dst_num_blocks) {
-		fprintf(stderr, "source LUN is bigger than destination (%"PRIu64" > %"PRIu64" sectors)\n", client.src_num_blocks, client.dst_num_blocks);
-		exit(10);
+	gettime_ret = clock_gettime(CLOCK_MONOTONIC, &start_time);
+	if (gettime_ret < 0) {
+		fprintf(stderr, "clock_gettime(CLOCK_MONOTONIC) failed\n");
 	}
 
 	if (client.use_xcopy) {
@@ -688,10 +731,10 @@ int main(int argc, char *argv[])
 	}
 
 	while (client.finished == 0) {
-		pfd[0].fd = iscsi_get_fd(client.src_iscsi);
-		pfd[0].events = iscsi_which_events(client.src_iscsi);
-		pfd[1].fd = iscsi_get_fd(client.dst_iscsi);
-		pfd[1].events = iscsi_which_events(client.dst_iscsi);
+		pfd[0].fd = iscsi_get_fd(client.src.iscsi);
+		pfd[0].events = iscsi_which_events(client.src.iscsi);
+		pfd[1].fd = iscsi_get_fd(client.dst.iscsi);
+		pfd[1].events = iscsi_which_events(client.dst.iscsi);
 
 		if (!pfd[0].events && !pfd[1].events) {
 			sleep(1);
@@ -699,23 +742,32 @@ int main(int argc, char *argv[])
 		}
 
 		if (poll(&pfd[0], 2, -1) < 0) {
-			printf("Poll failed");
+			fprintf(stderr, "Poll failed\n");
 			exit(10);
 		}
-		if (iscsi_service(client.src_iscsi, pfd[0].revents) < 0) {
-			printf("iscsi_service failed with : %s\n", iscsi_get_error(client.src_iscsi));
+		if (iscsi_service(client.src.iscsi, pfd[0].revents) < 0) {
+			fprintf(stderr, "iscsi_service failed with : %s\n", iscsi_get_error(client.src.iscsi));
 			break;
 		}
-		if (iscsi_service(client.dst_iscsi, pfd[1].revents) < 0) {
-			printf("iscsi_service failed with : %s\n", iscsi_get_error(client.dst_iscsi));
+		if (iscsi_service(client.dst.iscsi, pfd[1].revents) < 0) {
+			fprintf(stderr, "iscsi_service failed with : %s\n", iscsi_get_error(client.dst.iscsi));
 			break;
 		}
 	}
 
-	iscsi_logout_sync(client.src_iscsi);
-	iscsi_destroy_context(client.src_iscsi);
-	iscsi_logout_sync(client.dst_iscsi);
-	iscsi_destroy_context(client.dst_iscsi);
+	if (gettime_ret == 0) {
+		/* start_time is valid, so dump perf with a valid end_time */
+		gettime_ret = clock_gettime(CLOCK_MONOTONIC, &end_time);
+		if (gettime_ret == 0) {
+			show_perf(&start_time, &end_time, client.pos,
+				  client.src.blocksize);
+		}
+	}
+
+	iscsi_logout_sync(client.src.iscsi);
+	iscsi_destroy_context(client.src.iscsi);
+	iscsi_logout_sync(client.dst.iscsi);
+	iscsi_destroy_context(client.dst.iscsi);
 
 	return 0;
 }
