@@ -19,26 +19,31 @@
 
 #include <stdint.h>
 #include <time.h>
+#include <unistd.h>
 
 #if defined(_WIN32)
 #include <basetsd.h>
 #define ssize_t SSIZE_T
 #endif
 
+#include "iscsi.h"
+
 #ifdef __cplusplus
 extern "C" {
 #endif
 
-#ifndef discard_const
-#define discard_const(ptr) ((void *)((intptr_t)(ptr)))
-#endif
-
 #ifndef MIN
 #define MIN(a, b) (((a) < (b)) ? (a) : (b))
 #endif
 #ifndef MAX
 #define MAX(a, b) (((a) > (b)) ? (a) : (b))
 #endif
+#ifndef ARRAY_SIZE
+#define ARRAY_SIZE(x) (sizeof(x) / sizeof((x)[0]))
+#endif
+#ifndef DIV_ROUND_UP
+#define DIV_ROUND_UP(x, y) (((x) + (y) - 1) / (y))
+#endif
 
 #define ISCSI_RAW_HEADER_SIZE			48
 #define ISCSI_DIGEST_SIZE			 4
@@ -91,6 +96,7 @@ struct iscsi_context {
 
 	enum iscsi_session_type session_type;
 	unsigned char isid[6];
+	uint8_t rdma_ack_timeout;
 	uint32_t itt;
 	uint32_t cmdsn;
 	uint32_t min_cmdsn_waiting;
@@ -166,6 +172,8 @@ struct iscsi_context {
 	struct iscsi_context *old_iscsi;
 	int retry_cnt;
 	int no_ua_on_reconnect;
+	void (*fd_dup_cb)(struct iscsi_context *iscsi, void *opaque);
+	void *fd_dup_opaque;
 };
 
 #define ISCSI_PDU_IMMEDIATE		       0x40
@@ -286,10 +294,10 @@ void iscsi_pdu_set_datasn(struct iscsi_pdu *pdu, uint32_t datasn);
 void iscsi_pdu_set_bufferoffset(struct iscsi_pdu *pdu, uint32_t bufferoffset);
 void iscsi_cancel_pdus(struct iscsi_context *iscsi);
 int iscsi_pdu_add_data(struct iscsi_context *iscsi, struct iscsi_pdu *pdu,
-		       unsigned char *dptr, int dsize);
+		       const unsigned char *dptr, int dsize);
 int iscsi_queue_pdu(struct iscsi_context *iscsi, struct iscsi_pdu *pdu);
 int iscsi_add_data(struct iscsi_context *iscsi, struct iscsi_data *data,
-		   unsigned char *dptr, int dsize, int pdualignment);
+		   const unsigned char *dptr, int dsize, int pdualignment);
 
 struct scsi_task;
 void iscsi_pdu_set_cdb(struct iscsi_pdu *pdu, struct scsi_task *task);
@@ -325,13 +333,8 @@ int iscsi_process_reject(struct iscsi_context *iscsi,
 				struct iscsi_in_pdu *in);
 int iscsi_send_target_nop_out(struct iscsi_context *iscsi, uint32_t ttt, uint32_t lun);
 
-#if defined(_WIN32)
-void iscsi_set_error(struct iscsi_context *iscsi, const char *error_string,
-		     ...);
-#else
 void iscsi_set_error(struct iscsi_context *iscsi, const char *error_string,
 		     ...) __attribute__((format(printf, 2, 3)));
-#endif
 
 struct scsi_iovector *iscsi_get_scsi_task_iovector_in(struct iscsi_context *iscsi, struct iscsi_in_pdu *in);
 struct scsi_iovector *iscsi_get_scsi_task_iovector_out(struct iscsi_context *iscsi, struct iscsi_pdu *pdu);
@@ -359,7 +362,7 @@ void iscsi_decrement_iface_rr(void);
 		} \
 	} while (0)
 
-void
+void __attribute__((format(printf, 3, 4)))
 iscsi_log_message(struct iscsi_context *iscsi, int level, const char *format, ...);
 
 void
@@ -371,7 +374,7 @@ uint32_t iscsi_itt_post_increment(struct iscsi_context *iscsi);
 
 void iscsi_timeout_scan(struct iscsi_context *iscsi);
 
-void iscsi_reconnect_cb(struct iscsi_context *iscsi _U_, int status,
+void iscsi_reconnect_cb(struct iscsi_context *iscsi, int status,
                         void *command_data, void *private_data);
 
 struct iscsi_pdu *iscsi_tcp_new_pdu(struct iscsi_context *iscsi, size_t size);
@@ -397,6 +400,15 @@ typedef struct iscsi_transport {
 	int (*which_events)(struct iscsi_context *iscsi);
 } iscsi_transport;
 
+static inline int iscsi_dup2(struct iscsi_context *iscsi, int oldfd, int newfd)
+{
+	int ret = dup2(oldfd, newfd);
+	if ((ret >= 0) && iscsi->fd_dup_cb)
+		iscsi->fd_dup_cb(iscsi, iscsi->fd_dup_opaque);
+
+	return ret;
+}
+
 #ifdef __cplusplus
 }
 #endif
