@@ -46,7 +46,7 @@ struct connect_task {
 };
 
 static void
-iscsi_connect_cb(struct iscsi_context *iscsi, int status, void *command_data _U_,
+iscsi_connect_cb(struct iscsi_context *iscsi, int status, void *command_data,
 		 void *private_data);
 
 
@@ -144,7 +144,7 @@ iscsi_testunitready_cb(struct iscsi_context *iscsi, int status,
 }
 
 static void
-iscsi_login_cb(struct iscsi_context *iscsi, int status, void *command_data _U_,
+iscsi_login_cb(struct iscsi_context *iscsi, int status, void *command_data,
 	       void *private_data)
 {
 	struct connect_task *ct = private_data;
@@ -184,7 +184,7 @@ iscsi_login_cb(struct iscsi_context *iscsi, int status, void *command_data _U_,
 }
 
 static void
-iscsi_connect_cb(struct iscsi_context *iscsi, int status, void *command_data _U_,
+iscsi_connect_cb(struct iscsi_context *iscsi, int status, void *command_data,
 		 void *private_data)
 {
 	struct connect_task *ct = private_data;
@@ -276,8 +276,8 @@ void iscsi_defer_reconnect(struct iscsi_context *iscsi)
 	iscsi_cancel_pdus(iscsi);
 }
 
-void iscsi_reconnect_cb(struct iscsi_context *iscsi _U_, int status,
-                        void *command_data _U_, void *private_data _U_)
+void iscsi_reconnect_cb(struct iscsi_context *iscsi, int status,
+                        void *command_data, void *private_data)
 {
 	struct iscsi_context *old_iscsi;
 	int i;
@@ -378,7 +378,7 @@ void iscsi_reconnect_cb(struct iscsi_context *iscsi _U_, int status,
 	iscsi->pending_reconnect = 0;
 }
 
-int iscsi_reconnect(struct iscsi_context *iscsi)
+static int reconnect(struct iscsi_context *iscsi, int force)
 {
 	struct iscsi_context *tmp_iscsi;
 
@@ -397,7 +397,7 @@ int iscsi_reconnect(struct iscsi_context *iscsi)
 		return 0;
 	}
 
-	if (iscsi->old_iscsi && !iscsi->pending_reconnect) {
+	if (iscsi->old_iscsi && !iscsi->pending_reconnect && !force) {
 		return 0;
 	}
 
@@ -418,6 +418,14 @@ int iscsi_reconnect(struct iscsi_context *iscsi)
 		return -1;
 	}
 
+	/* default transport is initialized as TCP in iscsi_create_context,
+	   we have to overwrite transport in new iscsi as old iscsi.
+	 */
+	if (iscsi_init_transport(tmp_iscsi, iscsi->transport)) {
+		ISCSI_LOG(iscsi, 2, "failed to initializing transport for reconnection");
+		return -1;
+	}
+
 	ISCSI_LOG(iscsi, 2, "reconnect initiated");
 
 	iscsi_set_targetname(tmp_iscsi, iscsi->target_name);
@@ -446,6 +454,8 @@ int iscsi_reconnect(struct iscsi_context *iscsi)
 	tmp_iscsi->cache_allocations = iscsi->cache_allocations;
 	tmp_iscsi->scsi_timeout = iscsi->scsi_timeout;
 	tmp_iscsi->no_ua_on_reconnect = iscsi->no_ua_on_reconnect;
+	tmp_iscsi->fd_dup_cb = iscsi->fd_dup_cb;
+	tmp_iscsi->fd_dup_opaque = iscsi->fd_dup_opaque;
 
 	tmp_iscsi->reconnect_max_retries = iscsi->reconnect_max_retries;
 
@@ -454,9 +464,17 @@ int iscsi_reconnect(struct iscsi_context *iscsi)
 		for (i = 0; i < iscsi->smalloc_free; i++) {
 			iscsi_free(iscsi, iscsi->smalloc_ptrs[i]);
 		}
+		iscsi_free(iscsi, iscsi->opaque);
+
+		iscsi->old_iscsi->mallocs += iscsi->mallocs;
+		iscsi->old_iscsi->frees += iscsi->frees;
 		tmp_iscsi->old_iscsi = iscsi->old_iscsi;
 	} else {
 		tmp_iscsi->old_iscsi = malloc(sizeof(struct iscsi_context));
+		if (!tmp_iscsi->old_iscsi) {
+			free(tmp_iscsi);
+			return -1;
+		}
 		memcpy(tmp_iscsi->old_iscsi, iscsi, sizeof(struct iscsi_context));
 	}
 	memcpy(iscsi, tmp_iscsi, sizeof(struct iscsi_context));
@@ -465,3 +483,13 @@ int iscsi_reconnect(struct iscsi_context *iscsi)
 	return iscsi_full_connect_async(iscsi, iscsi->portal,
 	                                iscsi->lun, iscsi_reconnect_cb, NULL);
 }
+
+int iscsi_reconnect(struct iscsi_context *iscsi)
+{
+	return reconnect(iscsi, 0);
+}
+
+int iscsi_force_reconnect(struct iscsi_context *iscsi)
+{
+	return reconnect(iscsi, 1);
+}
