@@ -37,13 +37,7 @@ iscsi_discovery_async(struct iscsi_context *iscsi, iscsi_command_cb cb,
 		      void *private_data)
 {
 	struct iscsi_pdu *pdu;
-	char *str;
-
-	if (iscsi->session_type != ISCSI_SESSION_DISCOVERY) {
-		iscsi_set_error(iscsi, "Trying to do discovery on "
-				"non-discovery session.");
-		return -1;
-	}
+	const char *str;
 
 	pdu = iscsi_allocate_pdu(iscsi, ISCSI_PDU_TEXT_REQUEST,
 				 ISCSI_PDU_TEXT_RESPONSE,
@@ -68,9 +62,9 @@ iscsi_discovery_async(struct iscsi_context *iscsi, iscsi_command_cb cb,
 	iscsi_pdu_set_ttt(pdu, 0xffffffff);
 
 	/* sendtargets */
-	str = (char *)"SendTargets=All";
-	if (iscsi_pdu_add_data(iscsi, pdu, (unsigned char *)str, strlen(str)+1)
-	    != 0) {
+	str = "SendTargets=All";
+	if (iscsi_pdu_add_data(iscsi, pdu, (const unsigned char *)str,
+                               strlen(str) + 1) != 0) {
 		iscsi_set_error(iscsi, "Out-of-memory: pdu add data failed.");
 		iscsi->drv->free_pdu(iscsi, pdu);
 		return -1;
@@ -95,14 +89,14 @@ iscsi_free_discovery_addresses(struct iscsi_context *iscsi, struct iscsi_discove
 	while (addresses != NULL) {
 		struct iscsi_discovery_address *next = addresses->next;
 
-		iscsi_free(iscsi, discard_const(addresses->target_name));
+		iscsi_free(iscsi, addresses->target_name);
 		addresses->target_name = NULL;
 
 		while (addresses->portals != NULL) {
 			struct iscsi_target_portal *next_portal = addresses->portals->next;
 
-			iscsi_free(iscsi, discard_const(addresses->portals->portal));
-			iscsi_free(iscsi, discard_const(addresses->portals));
+			iscsi_free(iscsi, addresses->portals->portal);
+			iscsi_free(iscsi, addresses->portals);
 
 			addresses->portals = next_portal;
 		}
