@@ -218,7 +218,7 @@ iscsi_create_context(const char *initiator_name)
 
 	/* initalize transport of context */
 	if (iscsi_init_transport(iscsi, TCP_TRANSPORT)) {
-		iscsi_set_error(iscsi, "Failed allocating transport");
+		free(iscsi);
 		return NULL;
 	}
 
@@ -280,6 +280,10 @@ iscsi_create_context(const char *initiator_name)
 		iscsi_set_bind_interfaces(iscsi,getenv("LIBISCSI_BIND_INTERFACES"));
 	}
 
+	if (getenv("LIBISCSI_RDMA_ACK_TIMEOUT") != NULL) {
+		iscsi->rdma_ack_timeout = atoi(getenv("LIBISCSI_RDMA_ACK_TIMEOUT"));
+	}
+
 	/* iscsi->smalloc_size is the size for small allocations. this should be
 	   max(ISCSI_HEADER_SIZE, sizeof(struct iscsi_pdu), sizeof(struct iscsi_in_pdu))
 	   rounded up to the next power of 2. */
@@ -289,7 +293,8 @@ iscsi_create_context(const char *initiator_name)
 	while (iscsi->smalloc_size < required) {
 		iscsi->smalloc_size <<= 1;
 	}
-	ISCSI_LOG(iscsi,5,"small allocation size is %d byte", iscsi->smalloc_size);
+	ISCSI_LOG(iscsi, 5, "small allocation size is %u byte",
+                  (uint32_t)iscsi->smalloc_size);
 
 	ca = getenv("LIBISCSI_CACHE_ALLOCATIONS");
 	if (!ca || atoi(ca) != 0) {
@@ -393,9 +398,7 @@ iscsi_destroy_context(struct iscsi_context *iscsi)
 		return 0;
 	}
 
-	if (iscsi->fd != -1) {
-		iscsi_disconnect(iscsi);
-	}
+	iscsi_disconnect(iscsi);
 
 	iscsi_cancel_pdus(iscsi);
 
@@ -413,6 +416,8 @@ iscsi_destroy_context(struct iscsi_context *iscsi)
 		iscsi_free(iscsi, iscsi->smalloc_ptrs[i]);
 	}
 
+	iscsi_free(iscsi, iscsi->opaque);
+
 	if (iscsi->mallocs != iscsi->frees) {
 		ISCSI_LOG(iscsi,1,"%d memory blocks lost at iscsi_destroy_context() after %d malloc(s), %d realloc(s), %d free(s) and %d reused small allocations",iscsi->mallocs-iscsi->frees,iscsi->mallocs,iscsi->reallocs,iscsi->frees,iscsi->smallocs);
 	} else {
@@ -424,8 +429,6 @@ iscsi_destroy_context(struct iscsi_context *iscsi)
 		iscsi_destroy_context(iscsi->old_iscsi);
 	}
 
-	iscsi_free(iscsi, iscsi->opaque);
-
 	memset(iscsi, 0, sizeof(struct iscsi_context));
 	free(iscsi);
 
@@ -619,6 +622,8 @@ iscsi_parse_url(struct iscsi_context *iscsi, const char *url, int full)
 #ifdef HAVE_LINUX_ISER
 			} else if (!strcmp(key, "iser")) {
 				is_iser = 1;
+			} else if (!strcmp(key, "LIBISCSI_RDMA_ACK_TIMEOUT")) {
+				iscsi->rdma_ack_timeout = atoi(value);
 #endif
 			}
 			tmp = next;
@@ -826,3 +831,12 @@ iscsi_set_timeout(struct iscsi_context *iscsi, int timeout)
 	iscsi->scsi_timeout = timeout;
 	return 0;
 }
+
+void
+iscsi_set_fd_dup_cb(struct iscsi_context *iscsi,
+		    void (*cb)(struct iscsi_context *iscsi, void *opaque),
+		    void *opaque)
+{
+	iscsi->fd_dup_cb = cb;
+	iscsi->fd_dup_opaque = opaque;
+}
