@@ -40,7 +40,7 @@
 
 static void
 iscsi_scsi_response_cb(struct iscsi_context *iscsi, int status,
-		       void *command_data _U_, void *private_data)
+		       void *command_data, void *private_data)
 {
 	struct iscsi_scsi_cbdata *scsi_cbdata =
 	  (struct iscsi_scsi_cbdata *)private_data;
@@ -131,6 +131,7 @@ iscsi_send_data_out(struct iscsi_context *iscsi, struct iscsi_pdu *cmd_pdu,
 		if (iscsi_queue_pdu(iscsi, pdu) != 0) {
 			iscsi_set_error(iscsi, "Out-of-memory: failed to queue iscsi "
 				"scsi pdu.");
+			iscsi->drv->free_pdu(iscsi, pdu);
 			goto error;
 		}
 
@@ -271,7 +272,7 @@ iscsi_scsi_command_async(struct iscsi_context *iscsi, int lun,
 	iscsi_pdu_set_expxferlen(pdu, task->expxferlen);
 
 	/* cmdsn */
-	iscsi_pdu_set_cmdsn(pdu, iscsi->cmdsn++);
+	iscsi_pdu_set_cmdsn(pdu, iscsi->cmdsn);
 
 	/* cdb */
 	iscsi_pdu_set_cdb(pdu, task);
@@ -285,6 +286,7 @@ iscsi_scsi_command_async(struct iscsi_context *iscsi, int lun,
 		iscsi->drv->free_pdu(iscsi, pdu);
 		return -1;
 	}
+	iscsi->cmdsn++;
 
 	/* The F flag is not set. This means we haven't sent all the unsolicited
 	 * data yet. Sent as much as we are allowed as a train of DATA-OUT PDUs.
@@ -325,14 +327,21 @@ static void parse_sense_descriptors(struct scsi_sense *sense, const uint8_t *sb,
 	const unsigned char *p, *const end = sb + sb_len;
 
 	for (p = sb; p < end; p += p[1]) {
-		if (p[1] < 4) /* length */
-			break;
-		if (!(p[2] & 0x80)) /* VALID bit */
+		uint8_t addl_len = p[1];
+		if (addl_len < 4)
 			break;
 		switch (p[0]) {
+		case 0:
+			/* Information descriptor with VALID flag */
+			if (addl_len == 0x0a && p[2] & 0x80) {
+				sense->info_valid = 1;
+				sense->information = scsi_get_uint64(p + 4);
+			}
+			break;
 		case 2:
 			/* Sense key specific sense data descriptor */
-			parse_sense_spec(sense, p + 4);
+			if (addl_len == 0x06)
+				parse_sense_spec(sense, p + 4);
 			break;
 		}
 	}
@@ -346,6 +355,10 @@ void scsi_parse_sense_data(struct scsi_sense *sense, const uint8_t *sb)
 	case 0x71:
 		/* Fixed format */
 		sense->key  = sb[2] & 0x0f;
+		if (sb[0] & 0x80) {	/* VALID */
+			sense->info_valid = 1;
+			sense->information = scsi_get_uint32(sb + 3);
+		}
 		sense->ascq = scsi_get_uint16(&sb[12]);
 		parse_sense_spec(sense, sb + 15);
 		break;
@@ -2421,7 +2434,7 @@ iscsi_get_scsi_task_iovector_in(struct iscsi_context *iscsi, struct iscsi_in_pdu
 }
 
 struct scsi_iovector *
-iscsi_get_scsi_task_iovector_out(struct iscsi_context *iscsi _U_, struct iscsi_pdu *pdu)
+iscsi_get_scsi_task_iovector_out(struct iscsi_context *iscsi, struct iscsi_pdu *pdu)
 {
 	if (pdu->scsi_cbdata.task->iovector_out.iov == NULL) {
 		return NULL;
@@ -2662,11 +2675,27 @@ iscsi_scsi_get_task_from_pdu(struct iscsi_pdu *pdu)
 	return pdu->scsi_cbdata.task;
 }
 
+int iscsi_scsi_is_task_in_outqueue(struct iscsi_context *iscsi, struct scsi_task *task)
+{
+	struct iscsi_pdu *pdu;
+
+	for (pdu = iscsi->outqueue; pdu; pdu = pdu->next) {
+		if (pdu->itt == task->itt) {
+			return 1;
+		}
+	}
+
+	return 0;
+}
+
 int
 iscsi_scsi_cancel_task(struct iscsi_context *iscsi,
 		       struct scsi_task *task)
 {
 	struct iscsi_pdu *pdu;
+	struct iscsi_pdu *next_pdu;
+	uint32_t cmdsn_gap = 0;
+	int ret = -1;
 
 	for (pdu = iscsi->waitpdu; pdu; pdu = pdu->next) {
 		if (pdu->itt == task->itt) {
@@ -2679,22 +2708,45 @@ iscsi_scsi_cancel_task(struct iscsi_context *iscsi,
 			return 0;
 		}
 	}
-	for (pdu = iscsi->outqueue; pdu; pdu = pdu->next) {
+	for (pdu = iscsi->outqueue; pdu; pdu = next_pdu) {
+		next_pdu = pdu->next;
+
+		if (cmdsn_gap > 0) {
+			iscsi_pdu_set_cmdsn(pdu, pdu->cmdsn - cmdsn_gap);
+		}
+
 		if (pdu->itt == task->itt) {
 			ISCSI_LIST_REMOVE(&iscsi->outqueue, pdu);
 			if (pdu->callback) {
 				pdu->callback(iscsi, SCSI_STATUS_CANCELLED, NULL,
 				      pdu->private_data);
 			}
+			if (!(pdu->outdata.data[0] & ISCSI_PDU_IMMEDIATE) &&
+			    (pdu->outdata.data[0] & 0x3f) != ISCSI_PDU_DATA_OUT) {
+				iscsi->cmdsn--;
+				cmdsn_gap++;
+			}
 			iscsi->drv->free_pdu(iscsi, pdu);
-			return 0;
+			ret = 0;
+			if (!cmdsn_gap) {
+				break;
+			}
 		}
 	}
-	return -1;
+
+	if (iscsi->old_iscsi) {
+		return iscsi_scsi_cancel_task(iscsi->old_iscsi, task);
+	}
+
+	return ret;
 }
 
 void
 iscsi_scsi_cancel_all_tasks(struct iscsi_context *iscsi)
 {
 	iscsi_cancel_pdus(iscsi);
+
+	if (iscsi->old_iscsi) {
+		iscsi_cancel_pdus(iscsi->old_iscsi);
+	}
 }
