@@ -15,6 +15,10 @@
    along with this program; if not, see <http://www.gnu.org/licenses/>.
 */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
@@ -28,8 +32,8 @@
 #include "iser-private.h"
 #include "iscsi-private.h"
 #include "scsi-lowlevel.h"
-#include <pthread.h>
-#include <semaphore.h>
+#include <sys/eventfd.h>
+#include <limits.h>
 #include <poll.h>
 
 
@@ -42,7 +46,7 @@
   *
  */
 #define container_of(ptr, type, member) \
-    ((type *) ((uint8_t *)(ptr) - offsetof(type, member)))
+        ((type *)(void *)((uint8_t *)(ptr) - offsetof(type, member)))
 #endif
 
 
@@ -56,6 +60,8 @@ union socket_address {
 };
 
 static int cq_handle(struct iser_conn *iser_conn);
+static int iscsi_iser_revive_queued_pdus(struct iscsi_context *iscsi);
+static int iscsi_iser_cm_event(struct iscsi_context *iscsi);
 
 /*
  * iscsi_iser_get_fd() - Return completion queue
@@ -64,8 +70,7 @@ static int cq_handle(struct iser_conn *iser_conn);
 static int
 iscsi_iser_get_fd(struct iscsi_context *iscsi)
 {
-	struct iser_conn *iser_conn = iscsi->opaque;
-	return iser_conn->comp_channel->fd;
+	return iscsi->fd;
 }
 
 /*
@@ -78,7 +83,7 @@ iscsi_iser_get_fd(struct iscsi_context *iscsi)
  * Being used in QEMU iscsi block so we need compatability with TCP
  */
 static int
-iscsi_iser_which_events(struct iscsi_context *iscsi _U_)
+iscsi_iser_which_events(struct iscsi_context *iscsi)
 {
 	/* iSER is waiting to events from CQ that are always POLLIN */
 	return 1;
@@ -99,19 +104,251 @@ iscsi_iser_service(struct iscsi_context *iscsi, int revents)
 	int ret = 0;
 	struct iser_conn *iser_conn = iscsi->opaque;
 
+	if (iscsi->pending_reconnect) {
+		if (time(NULL) >= iscsi->next_reconnect) {
+			return iscsi_reconnect(iscsi);
+		} else {
+			if (iscsi->old_iscsi) {
+				return 0;
+			}
+		}
+	}
+
 	if (revents == POLLIN)
-		ret = cq_handle(iser_conn);
-	else {
+		ret = iscsi->is_connected ? cq_handle(iser_conn) : iscsi_iser_cm_event(iscsi);
+	else if (!revents) {
+		ret = iscsi_iser_cm_event(iscsi);
+	} else {
 		iscsi_set_error(iscsi, "revents is not POLLIN");
 		return -1;
 	}
 
 	if (ret) {
-		iscsi_set_error(iscsi, "CQ handle Failed");
-		return -1;
+		return iscsi_service_reconnect_if_loggedin(iscsi);
 	}
 
-	return 0;
+	return iscsi_iser_revive_queued_pdus(iscsi);
+}
+
+static inline int
+fls(int x)
+{
+    if (!x)
+        return 0;
+
+    return sizeof(int) * 8 - __builtin_clz(x);
+}
+
+static inline void
+iser_buf_chunk_tree_propagate(int8_t *tree, int pos, int level) {
+	int value;
+
+	for (pos >>= 1, level++; pos; pos >>= 1, level++) {
+		if (tree[pos << 1] == level - 1) {
+			if (tree[(pos << 1) | 1] == level - 1) {
+				value = level;
+			} else {
+				value = level - 1;
+			}
+		} else {
+			value = (tree[pos << 1] > tree[(pos << 1) | 1] ?
+			         tree[pos << 1] : tree[(pos << 1) | 1]);
+		}
+
+		if (value == tree[pos])
+			break;
+
+		tree[pos] = value;
+	}
+}
+
+static inline void
+iser_buf_chunk_free(struct iser_buf_chunk *chunk, void *ptr) {
+	int8_t *tree = chunk->tree;
+	int unit = ((unsigned char *)ptr - chunk->buf) >> DATA_BUFFER_UNIT_SHIFT;
+	int pos = unit + DATA_BUFFER_CHUNK_UNITS;
+	int level = 1;
+
+	for (; tree[pos]; pos >>= 1, level++) {
+	}
+
+	tree[pos] = level;
+	iser_buf_chunk_tree_propagate(tree, pos, level);
+}
+
+static inline void *
+iser_buf_chunk_alloc(struct iser_buf_chunk *chunk, int want) {
+	int8_t *tree = chunk->tree;
+	int pos, level, unit;
+	void *result;
+
+	/* satisfy ? */
+	if (tree[1] < want) {
+		return NULL;
+	}
+
+	/* lookup */
+	for (pos = 1, level = DATA_BUFFER_CHUNK_UNITS_SHIFT + 1; want < level; level--) {
+		pos = pos << 1;
+		pos = tree[pos] >= want ? pos : (pos | 1);
+	}
+
+	/* pick the node */
+	tree[pos] = 0;
+	unit = (pos << (level - 1)) - DATA_BUFFER_CHUNK_UNITS;
+	result = chunk->buf + (unit << DATA_BUFFER_UNIT_SHIFT);
+
+	/* propagate */
+	iser_buf_chunk_tree_propagate(tree, pos, level);
+
+	return result;
+}
+
+static void
+iser_tx_desc_free(struct iscsi_context *iscsi, struct iser_tx_desc *tx_desc)
+{
+	struct iser_conn *iser_conn = iscsi->opaque;
+	struct iser_buf_chunk *chunk = iser_conn->buf_chunk;
+
+	if (tx_desc->data_mr != NULL) {
+		for (; chunk != NULL; chunk = chunk->next) {
+			if (chunk->mr == tx_desc->data_mr) {
+				iser_buf_chunk_free(chunk, tx_desc->data_buff);
+				break;
+			}
+		}
+		if (chunk == NULL) {
+			ibv_dereg_mr(tx_desc->data_mr);
+			iscsi_free(iscsi, tx_desc->data_buff);
+		}
+	}
+
+	ISCSI_LIST_ADD(&iser_conn->tx_desc, tx_desc);
+}
+
+static struct iser_tx_desc *
+iser_tx_desc_alloc(struct iscsi_context *iscsi, size_t data_size) {
+	struct iser_conn *iser_conn = iscsi->opaque;
+	struct iser_tx_desc *tx_desc = iser_conn->tx_desc;
+	struct iser_buf_chunk **buf_chunk = &iser_conn->buf_chunk;
+	struct iser_buf_chunk *chunk;
+	int want, i;
+	void *buf;
+
+	if (tx_desc != NULL) {
+		ISCSI_LIST_REMOVE(&iser_conn->tx_desc, tx_desc);
+	} else {
+		tx_desc = iscsi_malloc(iscsi, sizeof(*tx_desc));
+		if (tx_desc == NULL) {
+			iscsi_set_error(iscsi, "Out-Of-Memory, failed to allocate data buffer");
+			return NULL;
+		}
+
+		tx_desc->hdr_mr = ibv_reg_mr(iser_conn->pd, tx_desc, ISER_HEADERS_LEN, IBV_ACCESS_LOCAL_WRITE);
+		if (tx_desc->hdr_mr == NULL) {
+			iscsi_free(iscsi, tx_desc);
+			iscsi_set_error(iscsi, "Failed to register data mr");
+			return NULL;
+		}
+	}
+
+	if (data_size == 0) {
+		tx_desc->data_buff = NULL;
+		tx_desc->data_mr = NULL;
+		return tx_desc;
+	} else if (data_size > DATA_BUFFER_CHUNK_SIZE) {
+		tx_desc->data_buff = iscsi_malloc(iscsi, data_size);
+		if (tx_desc->data_buff == NULL) {
+			iscsi_set_error(iscsi, "Out-Of-Memory, failed to allocate data buffer");
+			goto release;
+		}
+
+		tx_desc->data_mr = ibv_reg_mr(iser_conn->pd, tx_desc->data_buff, data_size,
+				IBV_ACCESS_LOCAL_WRITE | IBV_ACCESS_REMOTE_READ | IBV_ACCESS_REMOTE_WRITE);
+		if (tx_desc->data_mr == NULL) {
+			iscsi_free(iscsi, tx_desc->data_buff);
+			iscsi_set_error(iscsi, "Failed to register data mr");
+			goto release;
+		}
+		return tx_desc;
+	}
+
+	want = fls((data_size * 2 - 1) / DATA_BUFFER_UNIT_SIZE / 2) + 1;
+
+	for (; *buf_chunk != NULL; buf_chunk = &(*buf_chunk)->next) {
+		buf = iser_buf_chunk_alloc(*buf_chunk, want);
+		if (buf != NULL) {
+			tx_desc->data_buff = buf;
+			tx_desc->data_mr = (*buf_chunk)->mr;
+			return tx_desc;
+		}
+	}
+
+	chunk = iscsi_malloc(iscsi, sizeof(struct iser_buf_chunk));
+	if (chunk == NULL) {
+		iscsi_set_error(iscsi, "Out-Of-Memory, failed to allocate data buffer");
+		goto release;
+	}
+
+	chunk->buf = iscsi_malloc(iscsi, DATA_BUFFER_CHUNK_SIZE);
+	if (chunk == NULL) {
+		iscsi_set_error(iscsi, "Out-Of-Memory, failed to allocate data buffer");
+		goto free_chunk;
+	}
+
+	chunk->mr = ibv_reg_mr(iser_conn->pd, chunk->buf, DATA_BUFFER_CHUNK_SIZE,
+			IBV_ACCESS_LOCAL_WRITE | IBV_ACCESS_REMOTE_READ | IBV_ACCESS_REMOTE_WRITE);
+
+	if (chunk->mr == NULL) {
+		iscsi_set_error(iscsi, "Failed to register data mr");
+		goto free_chunk_buf;
+	}
+
+	for (i = 0; i < DATA_BUFFER_CHUNK_UNITS_SHIFT + 1; i++) {
+		memset(chunk->tree + (1 << i),
+		       DATA_BUFFER_CHUNK_UNITS_SHIFT - i + 1, (1 << i));
+	}
+	chunk->next = NULL;
+	*buf_chunk = chunk;
+
+	tx_desc->data_buff = iser_buf_chunk_alloc(chunk, want);
+	tx_desc->data_mr = chunk->mr;
+
+	return tx_desc;
+
+free_chunk_buf:
+	iscsi_free(iscsi, chunk->buf);
+
+free_chunk:
+	iscsi_free(iscsi, chunk);
+
+release:
+	ISCSI_LIST_ADD(&iser_conn->tx_desc, tx_desc);
+
+	return NULL;
+}
+
+static void
+iser_free_queued_pdu_tx_desc(struct iscsi_context *iscsi)
+{
+	struct iscsi_pdu *pdu;
+	struct iser_pdu *iser_pdu;
+
+	for (pdu = iscsi->waitpdu; pdu; pdu = pdu->next) {
+		iser_pdu = container_of(pdu, struct iser_pdu, iscsi_pdu);
+		if (iser_pdu->desc) {
+			iser_tx_desc_free(iscsi, iser_pdu->desc);
+			iser_pdu->desc = NULL;
+		}
+	}
+
+	for (pdu = iscsi->outqueue; pdu; pdu = pdu->next) {
+		iser_pdu = container_of(pdu, struct iser_pdu, iscsi_pdu);
+		if (iser_pdu->desc) {
+			iser_tx_desc_free(iscsi, iser_pdu->desc);
+			iser_pdu->desc = NULL;
+		}
+	}
 }
 
 /*
@@ -132,23 +369,6 @@ iser_free_rx_descriptors(struct iser_conn *iser_conn)
 	iscsi_free(iscsi, iser_conn->rx_descs);
 
 	iser_conn->rx_descs = NULL;
-
-	return;
-}
-
-/*
- * iser_free_login_buf() - freeing login buffer
- * @iser_conn:    ib connection context
- */
-static void
-iser_free_login_buf(struct iser_conn *iser_conn)
-{
-	struct iscsi_context *iscsi = iser_conn->cma_id->context;
-
-	iscsi_free(iscsi, iser_conn->login_buf);
-	iser_conn->login_buf = NULL;
-
-	return;
 }
 
 static void
@@ -157,13 +377,10 @@ iser_free_reg_mr(struct iser_conn *iser_conn)
 	struct iser_tx_desc *tx_desc = iser_conn->tx_desc;
 	struct iscsi_context *iscsi = iser_conn->cma_id->context;
 	struct iser_tx_desc *temp_tx_desc;
+	struct iser_buf_chunk *chunk, *temp_chunk;
 
 	while (tx_desc) {
 		ibv_dereg_mr(tx_desc->hdr_mr);
-		ibv_dereg_mr(tx_desc->data_mr);
-
-		if (tx_desc->data_buff)
-			iscsi_free(iscsi, tx_desc->data_buff);
 
 		temp_tx_desc = tx_desc;
 		tx_desc = tx_desc->next;
@@ -171,7 +388,15 @@ iser_free_reg_mr(struct iser_conn *iser_conn)
 	}
 	iser_conn->tx_desc = NULL;
 
-	return;
+	for (chunk = iser_conn->buf_chunk; chunk; ) {
+		ibv_dereg_mr(chunk->mr);
+		iscsi_free(iscsi, chunk->buf);
+
+		temp_chunk = chunk;
+		chunk = temp_chunk->next;
+		iscsi_free(iscsi, temp_chunk);
+	}
+	iser_conn->buf_chunk = NULL;
 }
 
 /*
@@ -191,16 +416,8 @@ iser_free_iser_conn_res(struct iser_conn *iser_conn, bool destroy)
 
 	if (destroy) {
 
-		if (iser_conn->cmthread) {
-			pthread_cancel(iser_conn->cmthread);
-			iser_conn->cmthread = 0;
-		}
-
 		iser_free_reg_mr(iser_conn);
 
-		if (iser_conn->login_buf)
-			iser_free_login_buf(iser_conn);
-
 		if (iser_conn->rx_descs) {
 			iser_free_rx_descriptors(iser_conn);
 			iser_conn->rx_descs = NULL;
@@ -212,7 +429,16 @@ iser_free_iser_conn_res(struct iser_conn *iser_conn, bool destroy)
 				iscsi_set_error(iscsi, "Failed to deregister login response mr");
 		}
 
+		if (iser_conn->login_resp_buf) {
+			iscsi_free(iscsi, iser_conn->login_resp_buf);
+			iser_conn->login_resp_buf = NULL;
+		}
+
 		if (iser_conn->cq) {
+			if (iser_conn->cq_nevents > 0) {
+				ibv_ack_cq_events(iser_conn->cq, iser_conn->cq_nevents);
+				iser_conn->cq_nevents = 0;
+			}
 			ret = ibv_destroy_cq(iser_conn->cq);
 			if (ret)
 				iscsi_set_error(iscsi, "Failed to destroy cq");
@@ -230,22 +456,21 @@ iser_free_iser_conn_res(struct iser_conn *iser_conn, bool destroy)
 				iscsi_set_error(iscsi, "Failed to deallocate pd");
 		}
 	}
-
-	return;
 }
 
 /*
  * iser_conn_release() - releasing ib resources
  *                       and destroying cm id
- * @iser_conn:    ib connection context
+ * @iscsi_context:    iscsi context
  */
 static void
-iser_conn_release(struct iser_conn *iser_conn)
+iser_conn_release(struct iscsi_context *iscsi)
 {
 	int ret;
-	struct iscsi_context *iscsi = iser_conn->cma_id->context;
+	struct iser_conn *iser_conn = iscsi->opaque;
 
-	iser_free_iser_conn_res(iser_conn,true);
+	iser_free_queued_pdu_tx_desc(iscsi);
+	iser_free_iser_conn_res(iser_conn, true);
 
 	if (iser_conn->cma_id) {
 		ret = rdma_destroy_id(iser_conn->cma_id);
@@ -255,7 +480,10 @@ iser_conn_release(struct iser_conn *iser_conn)
 		iser_conn->cma_id = NULL;
 	}
 
-	return;
+	if (iser_conn->cma_channel != NULL) {
+		rdma_destroy_event_channel(iser_conn->cma_channel);
+		iser_conn->cma_channel = NULL;
+	}
 }
 
 /*
@@ -268,14 +496,13 @@ iser_conn_terminate(struct iser_conn *iser_conn)
 	int ret;
 	struct iscsi_context *iscsi = iser_conn->cma_id->context;
 
-	if(iser_conn->cma_id) {
+	if(iser_conn->rdma_connect_sent) {
 		ret = rdma_disconnect(iser_conn->cma_id);
+		iser_conn->rdma_connect_sent = 0;
 		if (ret)
 			iscsi_set_error(iscsi, "Failed to disconnect, conn: 0x%p, err %d\n",
 					iser_conn, ret);
 	}
-
-	return;
 }
 
 /*
@@ -288,8 +515,14 @@ iscsi_iser_disconnect(struct iscsi_context *iscsi) {
 
 	struct iser_conn *iser_conn = iscsi->opaque;
 
-	iser_conn_terminate(iser_conn);
-	iser_conn_release(iser_conn);
+	if (iser_conn->cma_id) {
+		iser_conn_terminate(iser_conn);
+		iser_conn_release(iscsi);
+	}
+
+	if (iscsi->fd != -1) {
+		close(iscsi->fd);
+	}
 
 	iscsi->fd  = -1;
 	iscsi->is_connected = 0;
@@ -304,7 +537,7 @@ iscsi_iser_new_pdu(struct iscsi_context *iscsi, __attribute__((unused))size_t si
 	struct iscsi_pdu *pdu;
 	struct iser_pdu *iser_pdu;
 
-	iser_pdu = iscsi_zmalloc(iscsi, sizeof(*iser_pdu));
+	iser_pdu = iscsi_szmalloc(iscsi, sizeof(*iser_pdu));
 	pdu = &iser_pdu->iscsi_pdu;
 	pdu->indata.data = NULL;
 
@@ -323,6 +556,11 @@ iscsi_iser_free_pdu(struct iscsi_context *iscsi, struct iscsi_pdu *pdu)
 
 	iser_pdu = container_of(pdu, struct iser_pdu, iscsi_pdu);
 
+	if (iser_pdu->desc != NULL) {
+		iser_tx_desc_free(iscsi, iser_pdu->desc);
+		iser_pdu->desc = NULL;
+	}
+
 	if (pdu->outdata.size <= iscsi->smalloc_size) {
 		iscsi_sfree(iscsi, pdu->outdata.data);
 	} else {
@@ -342,7 +580,7 @@ iscsi_iser_free_pdu(struct iscsi_context *iscsi, struct iscsi_pdu *pdu)
 		iscsi->outqueue_current = NULL;
 	}
 
-	iscsi_free(iscsi, iser_pdu);
+	iscsi_sfree(iscsi, iser_pdu);
 }
 
 /**
@@ -430,6 +668,15 @@ iser_post_send(struct iser_conn *iser_conn, struct iser_tx_desc *tx_desc, bool s
 	return 0;
 }
 
+static inline int
+get_data_size(struct iser_pdu *iser_pdu)
+{
+	if (!iser_pdu->iscsi_pdu.scsi_cbdata.task)
+		return iser_pdu->iscsi_pdu.outdata.size - ISCSI_RAW_HEADER_SIZE;
+
+	return iser_pdu->iscsi_pdu.scsi_cbdata.task->expxferlen;
+}
+
 /*
  * iser_send_control() - sending iscsi pdu of type CONTROL
  *
@@ -458,28 +705,12 @@ iser_send_control(struct iser_conn *iser_conn, struct iser_pdu *iser_pdu) {
 		char* data = (char*)&iser_pdu->iscsi_pdu.outdata.data[ISCSI_RAW_HEADER_SIZE];
 		struct ibv_sge *tx_dsg = &tx_desc->tx_sg[1];
 
-		iser_conn->login_req_mr = ibv_reg_mr(iser_conn->pd, iser_conn->login_req_buf,
-							datalen , IBV_ACCESS_LOCAL_WRITE);
-		if (iser_conn->login_req_mr == NULL) {
-			iscsi_set_error(iscsi, "Failed Reg iser_conn->login_req_mr");
-			return -1;
-		}
+		memcpy(tx_desc->data_buff, data, datalen);
 
-		memcpy(iser_conn->login_req_buf, data, datalen);
-
-		tx_dsg->addr       = (uintptr_t)iser_conn->login_req_buf;
+		tx_dsg->addr       = (uintptr_t)tx_desc->data_buff;
 		tx_dsg->length     = datalen;
-		tx_dsg->lkey       = iser_conn->login_req_mr->lkey;
-		tx_desc->num_sge     = 2;
-	}
-
-	if (iser_pdu->iscsi_pdu.response_opcode == ISCSI_PDU_LOGIN_RESPONSE ||
-			iscsi->session_type == ISCSI_SESSION_DISCOVERY) {
-		ret = iser_post_recvl(iser_conn);
-		if (ret) {
-			iscsi_set_error(iscsi, "Failed Post Recv login");
-			return -1;
-		}
+		tx_dsg->lkey       = tx_desc->data_mr->lkey;
+		tx_desc->num_sge   = 2;
 	}
 
 	ret = iser_post_send(iser_conn, tx_desc, true);
@@ -499,12 +730,14 @@ iser_send_control(struct iser_conn *iser_conn, struct iser_pdu *iser_pdu) {
  * @iser_conn:       iser_connection context
  */
 static int
-iser_initialize_headers(struct iser_pdu *iser_pdu, struct iser_conn *iser_conn)
+iser_initialize_headers(struct iser_pdu *iser_pdu, struct iscsi_context *iscsi)
 {
 	struct iser_tx_desc *tx_desc;
 
-	tx_desc = iser_conn->tx_desc;
-	ISCSI_LIST_REMOVE(&iser_conn->tx_desc, tx_desc);
+	tx_desc = iser_tx_desc_alloc(iscsi, get_data_size(iser_pdu));
+	if (tx_desc == NULL) {
+		return -1;
+	}
 
 	iser_pdu->desc = tx_desc;
 
@@ -570,7 +803,8 @@ iser_prepare_write_cmd(struct iser_conn *iser_conn, struct iser_pdu *iser_pdu)
 	struct iser_hdr *hdr = &iser_pdu->desc->iser_header;
 	struct iscsi_context *iscsi = iser_conn->cma_id->context;
 	struct iser_tx_desc *tx_desc = iser_pdu->desc;
-	struct scsi_iovector *iovector = iscsi_get_scsi_task_iovector_out(iscsi, &iser_pdu->iscsi_pdu);
+	struct iscsi_pdu *iscsi_pdu = &iser_pdu->iscsi_pdu;
+	struct scsi_iovector *iovector = iscsi_get_scsi_task_iovector_out(iscsi, iscsi_pdu);
 	int i, offset = 0;
 
 	if (iovector == NULL) {
@@ -587,7 +821,20 @@ iser_prepare_write_cmd(struct iser_conn *iser_conn, struct iser_pdu *iser_pdu)
 
 	hdr->flags     |= ISER_WSV;
 	hdr->write_stag = htobe32((uint32_t)(tx_desc->data_mr->rkey));
-	hdr->write_va   = htobe64((intptr_t)(tx_desc->data_buff));
+
+	// ImmediateData
+	if (iscsi_pdu->payload_len > 0) {
+		struct ibv_sge *tx_dsg = &tx_desc->tx_sg[1];
+
+		tx_dsg->addr     = (uintptr_t)tx_desc->data_buff;
+		tx_dsg->length   = iscsi_pdu->payload_len;
+		tx_dsg->lkey     = tx_desc->data_mr->lkey;
+		tx_desc->num_sge = 2;
+
+		hdr->write_va = htobe64((intptr_t)(tx_desc->data_buff + iscsi_pdu->payload_len));
+	} else {
+		hdr->write_va = htobe64((intptr_t)(tx_desc->data_buff));
+	}
 
 	return 0;
 }
@@ -619,16 +866,6 @@ is_control_opcode(uint8_t opcode)
 	return is_control;
 }
 
-static int
-overflow_data_size(struct iser_pdu *iser_pdu)
-{
-	int data_size;
-
-	data_size = iser_pdu->iscsi_pdu.scsi_cbdata.task->expxferlen;
-
-	return (data_size > DATA_BUFFER_SIZE);
-}
-
 /*
  * iser_send_command() - sending iscsi pdu of type COMMAND
  *
@@ -649,11 +886,6 @@ iser_send_command(struct iser_conn *iser_conn, struct iser_pdu *iser_pdu)
 
 	iser_create_send_desc(iser_pdu);
 
-	if (overflow_data_size(iser_pdu)) {
-		iscsi_set_error(iscsi, "Libiscsi-iSER supports messages smaller than 512k\n");
-		return -1;
-	}
-
 	if (iser_pdu->desc->iscsi_header[1] & BHSSC_FLAGS_R) {
 		err = iser_prepare_read_cmd(iser_conn, iser_pdu);
 		if (err) {
@@ -666,7 +898,8 @@ iser_send_command(struct iser_conn *iser_conn, struct iser_pdu *iser_pdu)
 			iscsi_set_error(iscsi, "error in prepare write cmd\n");
 			return -1;
 		}
-	}
+	} else
+		iser_pdu->desc->data_dir = DATA_NONE;
 
 	err = iser_post_send(iser_conn, tx_desc, true);
 	if (err)
@@ -674,6 +907,66 @@ iser_send_command(struct iser_conn *iser_conn, struct iser_pdu *iser_pdu)
 	return 0;
 }
 
+static int
+iscsi_iser_send_pdu(struct iscsi_context *iscsi, struct iscsi_pdu *pdu) {
+	struct iser_pdu *iser_pdu;
+	struct iser_conn *iser_conn = iscsi->opaque;
+	uint8_t opcode;
+
+	iser_pdu = container_of(pdu, struct iser_pdu, iscsi_pdu);
+	opcode = pdu->outdata.data[0];
+
+	iscsi_pdu_set_expstatsn(pdu, iscsi->statsn + 1);
+	ISCSI_LIST_ADD_END(&iscsi->waitpdu, pdu);
+
+	/*
+         * because of async reconnection, before reconnecting successfully,
+	 * the argument 'iscsi' is the 'old_iscsi' with a empty connection.
+	 */
+	if (!iser_conn)
+		return 0;
+
+	if (iser_initialize_headers(iser_pdu, iscsi)) {
+		iscsi_set_error(iscsi, "initialize headers Failed\n");
+		return -1;
+	}
+
+	if (unlikely(is_control_opcode(opcode))) {
+		if (iser_send_control(iser_conn, iser_pdu)) {
+			iscsi_set_error(iscsi, "iser_send_command Failed\n");
+			return -1;
+		}
+	} else {
+		if (iser_send_command(iser_conn, iser_pdu)) {
+			iscsi_set_error(iscsi, "iser_send_command Failed\n");
+			return -1;
+		}
+	}
+
+	return 0;
+}
+
+static int
+iscsi_iser_revive_queued_pdus(struct iscsi_context *iscsi) {
+	struct iscsi_pdu *pdu;
+
+	while (iscsi->outqueue != NULL) {
+		if (iscsi_serial32_compare(iscsi->outqueue->cmdsn, iscsi->maxcmdsn) > 0) {
+			break;
+		}
+
+		pdu = iscsi->outqueue;
+		ISCSI_LIST_REMOVE(&iscsi->outqueue, pdu);
+
+		if (iscsi_iser_send_pdu(iscsi, pdu) < 0) {
+			ISCSI_LIST_ADD(&iscsi->outqueue, pdu);
+			return -1;
+		}
+	}
+
+	return 0;
+}
+
 
 /*
  * iser_queue_pdu() - sending iscsi pdu
@@ -683,46 +976,32 @@ iser_send_command(struct iser_conn *iser_conn, struct iser_pdu *iser_pdu)
  *
  * Notes:
  * Need to be compatible to TCP which has real queue,
- * in iSER every queue pdu already sends all pdu (post_send)
+ * in iSER pdus with cmdsn not exceeds maxcmdsn are already sent.
  */
 static int
 iscsi_iser_queue_pdu(struct iscsi_context *iscsi, struct iscsi_pdu *pdu) {
-
-	struct iser_pdu *iser_pdu;
-	struct iser_conn *iser_conn = iscsi->opaque;
-	uint8_t opcode;
-
 	if (pdu == NULL) {
 		iscsi_set_error(iscsi, "trying to queue NULL pdu");
 		return -1;
 	}
 
-	iser_pdu = container_of(pdu, struct iser_pdu, iscsi_pdu);
-	opcode = pdu->outdata.data[0];
-
-	iscsi_pdu_set_expstatsn(pdu, iscsi->statsn + 1);
-	ISCSI_LIST_ADD_END(&iscsi->waitpdu, pdu);
-
-	if (iser_initialize_headers(iser_pdu, iser_conn)) {
-		iscsi_set_error(iscsi, "initialize headers Failed\n");
+	if (pdu->outdata.data[0] == ISCSI_PDU_NOP_OUT &&
+		iscsi_iser_cm_event(iscsi) != 0) {
+		iscsi_service_reconnect_if_loggedin(iscsi);
 		return -1;
 	}
 
-	if (unlikely(is_control_opcode(opcode))) {
-		if (iser_send_control(iser_conn, iser_pdu)) {
-			iscsi_set_error(iscsi, "iser_send_command Failed\n");
-			return -1;
-		}
-	} else {
-                if (iser_send_command(iser_conn, iser_pdu)) {
-			iscsi_set_error(iscsi, "iser_send_command Failed\n");
-			return -1;
-		}
+	if (iscsi->outqueue != NULL ||
+		(iscsi_serial32_compare(pdu->cmdsn, iscsi->maxcmdsn) > 0
+		 && !(pdu->outdata.data[0] & ISCSI_PDU_IMMEDIATE))) {
+		iscsi_add_to_outqueue(iscsi, pdu);
+		return 0;
 	}
 
-	return 0;
+	return iscsi_iser_send_pdu(iscsi, pdu);
 }
 
+
 /*
  * iser_create_iser_conn_res() - creating ib connections resources
  *
@@ -754,6 +1033,40 @@ static int iser_create_iser_conn_res(struct iser_conn *iser_conn) {
 	return ret;
 }
 
+void iscsi_set_rdma_ack_timetout(struct iscsi_context *iscsi, unsigned char value)
+{
+	iscsi->rdma_ack_timeout = value;
+	ISCSI_LOG(iscsi, 2, "RDMA ACK TIMEOUT will be set to %d on next rdma qp creation", value);
+}
+
+/*
+ * iser_rdma_set_option() - set rdma options
+ * currently support RDMA ack timeout.
+ * @cma_id:    connection manager id
+ * @iscsi:   iscsi context
+ */
+int iser_rdma_set_option(struct rdma_cm_id *cma_id, struct iscsi_context *iscsi) {
+	int ret = 0;
+	uint8_t timeout = iscsi->rdma_ack_timeout;
+
+	if (timeout) {
+#ifdef HAVE_RDMA_ACK_TIMEOUT
+		/* calculate according to the formula 4.096 * 2^(timeout) usec.
+		   Ex, 8 means 1048.576 usec (0.00104 sec) */
+		ret = rdma_set_option(cma_id, RDMA_OPTION_ID, RDMA_OPTION_ID_ACK_TIMEOUT, &timeout, sizeof(timeout));
+		if (ret) {
+			ISCSI_LOG(iscsi, 1, "failed to set RDMA ACK TIMEOUT: %s", strerror(errno));
+		} else {
+			ISCSI_LOG(iscsi, 3, "RDMA ACK TIMEOUT set to %d", timeout);
+		}
+#else
+		ISCSI_LOG(iscsi, 1, "failed to set RDMA ACK TIMEOUT(%d) for cma_id(%p), please update your kernel&librdmacm", timeout, cma_id);
+#endif
+	}
+
+	return ret;
+}
+
 /*
  * iser_addr_handler() - handles RDMA_CM_EVENT_ADDR_RESOLVED
  *                       event in rdma_cm
@@ -765,6 +1078,8 @@ static int iser_addr_handler(struct rdma_cm_id *cma_id) {
 	struct iser_conn *iser_conn = iscsi->opaque;
 	int ret, flags;
 
+	iser_rdma_set_option(cma_id, iscsi);
+
 	ret = rdma_resolve_route(cma_id, 1000);
 	if (ret) {
 		iscsi_set_error(iscsi, "Failed resolving address\n");
@@ -798,26 +1113,24 @@ static int iser_addr_handler(struct rdma_cm_id *cma_id) {
 		iscsi_set_error(iscsi, "Failed to create cq\n");
 		goto pd_error;
 	}
+	iser_conn->cq_nevents = 0;
 
 	if (ibv_req_notify_cq(iser_conn->cq, 0)) {
 		iscsi_set_error(iscsi, "ibv_req_notify_cq failed\n");
 		goto cq_error;
 	}
 
-	iser_conn->login_buf = iscsi_malloc(iscsi, ISCSI_DEF_MAX_RECV_SEG_LEN + ISER_RX_LOGIN_SIZE);
-	if (!iser_conn->login_buf) {
-		iscsi_set_error(iscsi, "Failed to allocate memory for login_buf\n");
-		iscsi_free(iscsi, iser_conn->login_buf);
+	iser_conn->login_resp_buf = iscsi_malloc(iscsi, ISER_RX_LOGIN_SIZE);
+	if (!iser_conn->login_resp_buf) {
+		iscsi_set_error(iscsi, "Failed to allocate memory for login_resp_buf\n");
 		goto cq_error;
 	}
 
-	iser_conn->login_req_buf = iser_conn->login_buf;
-	iser_conn->login_resp_buf = iser_conn->login_buf + ISCSI_DEF_MAX_RECV_SEG_LEN;
 	iser_conn->login_resp_mr = ibv_reg_mr(iser_conn->pd, iser_conn->login_resp_buf,
 					    ISER_RX_LOGIN_SIZE, IBV_ACCESS_LOCAL_WRITE);
 	if(!iser_conn->login_resp_mr) {
 		iscsi_set_error(iscsi, "Failed to reg login_resp_mr\n");
-		iscsi_free(iscsi, iser_conn->login_buf);
+		iscsi_free(iscsi, iser_conn->login_resp_buf);
 		goto cq_error;
 	}
 
@@ -866,6 +1179,7 @@ static int iser_route_handler(struct rdma_cm_id *cma_id) {
 		iscsi_set_error(iscsi, "conn %p failure connecting: %d", iser_conn, ret);
 		return -1;
 	}
+	iser_conn->rdma_connect_sent = 1;
 	return ret;
 
 login_mr_error:
@@ -972,46 +1286,6 @@ iser_post_recvm(struct iser_conn *iser_conn, int count)
 	return ret;
 }
 
-static int
-iser_reg_mr(struct iser_conn *iser_conn)
-{
-	int i;
-	struct iscsi_context *iscsi = iser_conn->cma_id->context;
-	struct iser_tx_desc *tx_desc;
-
-	for (i = 0 ; i < NUM_MRS ; i++) {
-
-			tx_desc = iscsi_malloc(iscsi, sizeof(*tx_desc));
-			if (tx_desc == NULL) {
-				iscsi_set_error(iscsi, "Out-Of-Memory, failed to allocate data buffer");
-				return -1;
-			}
-
-			tx_desc->hdr_mr = ibv_reg_mr(iser_conn->pd, tx_desc, ISER_HEADERS_LEN, IBV_ACCESS_LOCAL_WRITE);
-			if (tx_desc->hdr_mr == NULL) {
-				iscsi_set_error(iscsi, "Failed to register data mr");
-				return -1;
-			}
-
-			tx_desc->data_buff = iscsi_malloc(iscsi, DATA_BUFFER_SIZE);
-			if (tx_desc->data_buff == NULL) {
-				iscsi_set_error(iscsi, "Out-Of-Memory, failed to allocate data buffer");
-				return -1;
-			}
-
-			tx_desc->data_mr = ibv_reg_mr(iser_conn->pd, tx_desc->data_buff, DATA_BUFFER_SIZE,
-							IBV_ACCESS_LOCAL_WRITE | IBV_ACCESS_REMOTE_READ | IBV_ACCESS_REMOTE_WRITE);
-			if (tx_desc->data_mr == NULL) {
-				iscsi_set_error(iscsi, "Failed to register data mr");
-				return -1;
-			}
-
-			ISCSI_LIST_ADD_END(&iser_conn->tx_desc, tx_desc);
-	}
-
-	return 0;
-}
-
 /**
  * iser_rcv_completion() - handling and processing receive completion
  *
@@ -1027,34 +1301,22 @@ static int
 iser_rcv_completion(struct iser_rx_desc *rx_desc,
 		    struct iser_conn *iser_conn)
 {
-	struct iscsi_in_pdu *in = NULL;
+	struct iscsi_in_pdu in;
 	int empty, err;
 	struct iscsi_context *iscsi = iser_conn->cma_id->context;
 
-	in = iscsi_malloc(iscsi, sizeof(*in));
+	in.hdr = (unsigned char*)rx_desc->iscsi_header;
+	in.data_pos = iscsi_get_pdu_data_size(&in.hdr[0]);
+	in.data = (unsigned char*)rx_desc->data;
 
-	if ((unsigned char *)rx_desc == iser_conn->login_resp_buf)
-		if (iscsi->session_type == ISCSI_SESSION_NORMAL) {
-			if(iser_alloc_rx_descriptors(iser_conn,255)) {
-				iscsi_set_error(iscsi, "iser_alloc_rx_descriptors Failed\n");
-				err = -1;
-				goto error;
-			}
-			err = iser_post_recvm(iser_conn, ISER_MIN_POSTED_RX);
-			if (err) {
-				err = -1;
-				goto error;
-			}
-		}
-	in->hdr = (unsigned char*)rx_desc->iscsi_header;
-	in->data_pos = iscsi_get_pdu_data_size(&in->hdr[0]);
-	in->data = (unsigned char*)rx_desc->data;
-
-	enum iscsi_opcode opcode = in->hdr[0] & 0x3f;
-	uint32_t itt = scsi_get_uint32(&in->hdr[16]);
+	enum iscsi_opcode opcode = in.hdr[0] & 0x3f;
+	uint32_t itt = scsi_get_uint32(&in.hdr[16]);
 
 	if (opcode == ISCSI_PDU_NOP_IN && itt == 0xffffffff)
-		goto nop_target;
+		goto no_waitpdu;
+
+	if (opcode == ISCSI_PDU_ASYNC_MSG)
+		goto no_waitpdu;
 
 	struct iscsi_pdu *iscsi_pdu;
 	struct iser_pdu *iser_pdu;
@@ -1087,9 +1349,7 @@ iser_rcv_completion(struct iser_rx_desc *rx_desc,
 		}
 	}
 
-	ISCSI_LIST_ADD_END(&iser_conn->tx_desc, iser_pdu->desc);
-
-nop_target:
+no_waitpdu:
 	/* decrementing conn->post_recv_buf_count only --after-- freeing the   *
 	 * task eliminates the need to worry on tasks which are completed in   *
 	 * parallel to the execution of iser_conn_term. So the code that waits *
@@ -1101,17 +1361,26 @@ nop_target:
 		if (empty >= iser_conn->min_posted_rx) {
 			err = iser_post_recvm(iser_conn, empty);
 			if (err) {
-				err = -1;
-				goto error;
+				return -1;
 			}
 		}
+	} else if (iscsi->is_loggedin) {
+		if(iser_alloc_rx_descriptors(iser_conn, 255)) {
+			iscsi_set_error(iscsi, "iser_alloc_rx_descriptors Failed\n");
+			return -1;
+		}
+		err = iser_post_recvm(iser_conn, ISER_MIN_POSTED_RX);
+		if (err) {
+			return -1;
+		}
+	} else {
+		if (iser_post_recvl(iser_conn)) {
+			iscsi_set_error(iscsi, "Failed Post Recv login");
+			return -1;
+		}
 	}
 
-	err = iscsi_process_pdu(iscsi, in);
-
-error:
-	iscsi_free(iscsi, in);
-	return err;
+	return iscsi_process_pdu(iscsi, &in);
 }
 
 /**
@@ -1122,8 +1391,8 @@ error:
  *
  */
 static int
-iser_snd_completion(struct iser_tx_desc *tx_desc _U_,
-		    struct iser_conn *iser_conn _U_)
+iser_snd_completion(struct iser_tx_desc *tx_desc,
+		    struct iser_conn *iser_conn)
 {
 	return 0;
 }
@@ -1213,16 +1482,28 @@ static int cq_handle(struct iser_conn *iser_conn)
 	int ret;
 	struct iscsi_context *iscsi = iser_conn->cma_id->context;
 
-	ibv_get_cq_event(iser_conn->comp_channel, &iser_conn->cq, &ev_ctx);
+	ret = ibv_get_cq_event(iser_conn->comp_channel, &iser_conn->cq, &ev_ctx);
+	if (ret) {
+		if (errno != EAGAIN && errno != EWOULDBLOCK) {
+			iscsi_set_error(iscsi, "failed get cq event %s", strerror(errno));
+			return -1;
+		}
+		goto out;
+	}
+
 	ret = ibv_req_notify_cq(iser_conn->cq, 0);
 
-	/* FIXME: aggregate ack cq event for efficiency */
-	ibv_ack_cq_events(iser_conn->cq, 1);
+	if (++iser_conn->cq_nevents >= INT_MAX) {
+		ibv_ack_cq_events(iser_conn->cq, iser_conn->cq_nevents);
+		iser_conn->cq_nevents = 0;
+	}
+
 	if (ret) {
 		iscsi_set_error(iscsi, "failed notify or ack CQ");
 		return -1;
 	}
 
+out:
 	ret = cq_event_handler(iser_conn);
 	if (ret) {
 		iscsi_set_error(iscsi, "failed CQ handler");
@@ -1244,10 +1525,19 @@ static int iser_connected_handler(struct rdma_cm_id *cma_id) {
 	struct iscsi_context *iscsi = cma_id->context;
 	struct iser_conn *iser_conn = iscsi->opaque;
 
+	if (iscsi_dup2(iscsi, iser_conn->comp_channel->fd, iscsi->fd) == -1) {
+		return -1;
+	}
+
 	iser_conn->post_recv_buf_count = 0;
+	iscsi->is_connected = 1;
 
-	return iser_reg_mr(iser_conn);
+	if (iser_post_recvl(iser_conn)) {
+		iscsi_set_error(iscsi, "Failed Post Recv login");
+		return -1;
+	}
 
+	return 0;
 }
 
 /*
@@ -1259,8 +1549,7 @@ static int iser_connected_handler(struct rdma_cm_id *cma_id) {
  *
  */
 static int
-iser_cma_handler(struct iser_conn *iser_conn,struct rdma_cm_id *cma_id, struct rdma_cm_event *event) {
-
+iser_cma_handler(struct rdma_cm_id *cma_id, struct rdma_cm_event *event) {
 	int ret = 0;
 
 	switch(event->event) {
@@ -1273,66 +1562,64 @@ iser_cma_handler(struct iser_conn *iser_conn,struct rdma_cm_id *cma_id, struct r
 			break;
 		case RDMA_CM_EVENT_ESTABLISHED:
 			ret = iser_connected_handler(cma_id);
-			if(ret)
-				iser_conn->conn_state = CONN_ERROR;
-			else
-				iser_conn->conn_state = CONN_ESTABLISHED;
-			sem_post(&iser_conn->sem_connect);
 			break;
 		case RDMA_CM_EVENT_ADDR_ERROR:
 		case RDMA_CM_EVENT_ROUTE_ERROR:
 		case RDMA_CM_EVENT_CONNECT_ERROR:
 		case RDMA_CM_EVENT_UNREACHABLE:
 		case RDMA_CM_EVENT_REJECTED:
-			iser_conn->conn_state = CONN_ERROR;
 			ret = -1;
-			sem_post(&iser_conn->sem_connect);
 			break;
 		case RDMA_CM_EVENT_DISCONNECTED:
 		case RDMA_CM_EVENT_ADDR_CHANGE:
 		case RDMA_CM_EVENT_TIMEWAIT_EXIT:
-			iser_conn->conn_state = CONN_DISCONNECTED;
 			ret = -1;
-			sem_post(&iser_conn->sem_connect);
 			break;
 		default:
-			iser_conn->conn_state = CONN_ERROR;
 			ret = -1;
-			sem_post(&iser_conn->sem_connect);
 			break;
 	}
 	return ret;
 }
 
-/*
- * iser_connected_handler() - thread to catch rdma cm events
- *
- * @arg:    ib connection context
- *
- * Notes:
- * Need to check if copying event is necessery
- */
-static void *cm_thread(void *arg)
-{
-	struct iser_conn *iser_conn = arg;
-	struct rdma_cm_event event_copy;
+static int
+iscsi_iser_cm_event(struct iscsi_context *iscsi) {
+	struct iser_conn *iser_conn = iscsi->opaque;
+	struct rdma_event_channel *channel = iser_conn->cma_channel;
+	struct rdma_cm_event *event;
 	int ret;
-	struct iscsi_context *iscsi = iser_conn->cma_id->context;
 
-	while (1) {
-		ret = rdma_get_cm_event(iser_conn->cma_channel, &iser_conn->cma_event);
+	while (rdma_get_cm_event(channel, &event) == 0) {
+		ret = iser_cma_handler(iser_conn->cma_id, event);
+		rdma_ack_cm_event(event);
+
 		if (ret) {
-			iscsi_set_error(iscsi, "Failed to get RDMA-CM Event\n");
-			pthread_exit(NULL);
+			if (iscsi->socket_status_cb != NULL) {
+				/* connect failed, cleanup itself */
+				if (iser_conn->cma_id) {
+					iser_conn_terminate(iser_conn);
+					iser_conn_release(iscsi);
+					iser_conn->cma_id = NULL;
+				}
+
+				iscsi->socket_status_cb(iscsi, SCSI_STATUS_ERROR, NULL, iscsi->connect_data);
+				iscsi->socket_status_cb = NULL;
+			}
+			return -1;
 		}
-		memcpy(&event_copy, iser_conn->cma_event, sizeof(struct rdma_cm_event));
-		ret = iser_cma_handler(iser_conn, iser_conn->cma_id, &event_copy);
-		rdma_ack_cm_event(iser_conn->cma_event);
-		if (ret) {
-			iscsi_set_error(iscsi, "Failed to handle event\n");
-			pthread_exit(NULL);
+
+		if (iscsi->is_connected && iscsi->socket_status_cb != NULL) {
+			iscsi->socket_status_cb(iscsi, SCSI_STATUS_GOOD, NULL, iscsi->connect_data);
+			iscsi->socket_status_cb = NULL;
 		}
 	}
+
+	if (errno != EAGAIN && errno != EWOULDBLOCK) {
+		iscsi_set_error(iscsi, "Failed to get RDMA-CM Event\n");
+		return -1;
+	}
+
+	return 0;
 }
 
 /*
@@ -1350,9 +1637,33 @@ static int
 iscsi_iser_connect(struct iscsi_context *iscsi, union socket_address *sa,__attribute__((unused)) int ai_family) {
 
 	struct iser_conn *iser_conn = iscsi->opaque;
-	int ret;
+	int flag;
 
-	sem_init(&iser_conn->sem_connect, 0, 0);
+	if (iscsi->old_iscsi && iscsi->fd != iscsi->old_iscsi->fd) {
+		struct iscsi_context *old_iscsi = iscsi->old_iscsi;
+		struct iser_conn *old_iser_conn = old_iscsi->opaque;
+
+		iscsi->fd = old_iscsi->fd;
+
+		if (old_iser_conn) {
+			if (old_iser_conn->cma_id) {
+				iser_conn_terminate(old_iser_conn);
+				iser_conn_release(old_iscsi);
+			}
+
+			iscsi_free(old_iscsi, old_iser_conn);
+			old_iscsi->opaque = NULL;
+		}
+	}
+
+	/* create mock file descriptor */
+	if (iscsi->fd == -1) {
+		iscsi->fd = eventfd(0, EFD_CLOEXEC);
+
+		if (iscsi->fd < 0) {
+			return -1;
+		}
+	}
 
 	iser_conn->cma_channel = rdma_create_event_channel();
 
@@ -1361,49 +1672,40 @@ iscsi_iser_connect(struct iscsi_context *iscsi, union socket_address *sa,__attri
 		return -1;
 	}
 
-	if (rdma_create_id(iser_conn->cma_channel, &iser_conn->cma_id, (void *)iscsi, RDMA_PS_TCP)) {
-		iscsi_set_error(iscsi, "Failed create channel_id");
-		return -1;
+	flag = fcntl(iser_conn->cma_channel->fd, F_GETFL);
+	if (fcntl(iser_conn->cma_channel->fd, F_SETFL, flag | O_NONBLOCK) < 0) {
+		iscsi_set_error(iscsi, "Cannot set event channel to non blocking");
+		goto error;
 	}
 
-	ret = pthread_create(&iser_conn->cmthread, NULL, cm_thread, iser_conn);
-	if(ret) {
-		iscsi_set_error(iscsi, "Failed create Connection Manager Thread");
-		return -1;
+	if (rdma_create_id(iser_conn->cma_channel, &iser_conn->cma_id, (void *)iscsi, RDMA_PS_TCP)) {
+		iscsi_set_error(iscsi, "Failed create channel_id");
+		goto error;
+	}
+
+	if (iscsi_dup2(iscsi, iser_conn->cma_channel->fd, iscsi->fd) < 0) {
+		iscsi_set_error(iscsi, "Failed dup event channel fd");
+		goto error;
 	}
 
 	if(rdma_resolve_addr(iser_conn->cma_id, NULL, &sa->sa, 2000)) {
 		iscsi_set_error(iscsi, "Failed resolve address");
-		return -1;
+		goto error;
 	}
 
-	sem_wait(&iser_conn->sem_connect);
-
-	switch(iser_conn->conn_state) {
-
-		case CONN_ERROR:
-			iscsi_set_error(iscsi, "Conn Error event");
-			return -1;
-
-		case CONN_DISCONNECTED:
-			iscsi_set_error(iscsi, "Conn disconnected event");
-			return -1;
-
-		case CONN_ESTABLISHED:
-			break;
-
-		default:
-			iscsi_set_error(iscsi, "Unknown State of connection");
-			return -1;
-
-	}
-
-
-	iscsi->is_connected = 1;
-	iscsi->socket_status_cb(iscsi, SCSI_STATUS_GOOD, NULL, iscsi->connect_data);
-	iscsi->socket_status_cb = NULL;
-
 	return 0;
+
+error:
+	if (iser_conn->cma_id) {
+		rdma_destroy_id(iser_conn->cma_id);
+		iser_conn->cma_id = NULL;
+	}
+	if (iser_conn->cma_channel) {
+		rdma_destroy_event_channel(iser_conn->cma_channel);
+		iser_conn->cma_channel = NULL;
+	}
+
+	return -1;
 }
 
 static iscsi_transport iscsi_transport_iser = {
@@ -1420,11 +1722,16 @@ static iscsi_transport iscsi_transport_iser = {
 void iscsi_init_iser_transport(struct iscsi_context *iscsi)
 {
 	iscsi->drv = &iscsi_transport_iser;
-	iscsi->opaque = iscsi_malloc(iscsi, sizeof(struct iser_conn));
+	iscsi->opaque = iscsi_zmalloc(iscsi, sizeof(struct iser_conn));
 	iscsi->transport = ISER_TRANSPORT;
 	/* Update iSCSI params as per iSER transport */
 	iscsi->initiator_max_recv_data_segment_length = ISCSI_DEF_MAX_RECV_SEG_LEN;
 	iscsi->target_max_recv_data_segment_length = ISCSI_DEF_MAX_RECV_SEG_LEN;
+
+	/* ensure smalloc_size is enough for iser_pdu */
+	while (iscsi->smalloc_size < sizeof(struct iser_pdu)) {
+		iscsi->smalloc_size <<= 1;
+	}
 }
 
 #endif
