@@ -48,6 +48,7 @@
 #include "iscsi-private.h"
 #include "scsi-lowlevel.h"
 #include "slist.h"
+#include "utils.h"
 
 /* This adds 32-bit serial comparision as defined in RFC1982.
  * It returns 0 for equality, 1 if s1 is greater than s2 and
@@ -78,23 +79,112 @@ iscsi_itt_post_increment(struct iscsi_context *iscsi) {
 	return old_itt;
 }
 
+static const char *
+iscsi_opcode_str(int opcode)
+{
+	static struct iscsi_value_string opcode_strings[] = {
+		{ ISCSI_PDU_NOP_OUT,
+			"NOP_OUT" },
+		{ ISCSI_PDU_SCSI_REQUEST,
+			"SCSI_REQUEST" },
+		{ ISCSI_PDU_SCSI_TASK_MANAGEMENT_REQUEST,
+			"SCSI_TASK_MANAGEMENT_REQUEST" },
+		{ ISCSI_PDU_LOGIN_REQUEST,
+			"LOGIN_REQUEST" },
+		{ ISCSI_PDU_TEXT_REQUEST,
+			"TEXT_REQUEST" },
+		{ ISCSI_PDU_DATA_OUT,
+			"DATA_OUT" },
+		{ ISCSI_PDU_LOGOUT_REQUEST,
+			"LOGOUT_REQUEST" },
+		{ ISCSI_PDU_NOP_IN,
+			"NOP_IN" },
+		{ ISCSI_PDU_SCSI_RESPONSE,
+			"SCSI_RESPONSE" },
+		{ ISCSI_PDU_SCSI_TASK_MANAGEMENT_RESPONSE,
+			"SCSI_TASK_MANAGEMENT_RESPONSE" },
+		{ ISCSI_PDU_LOGIN_RESPONSE,
+			"LOGIN_RESPONSE" },
+		{ ISCSI_PDU_TEXT_RESPONSE,
+			"TEXT_RESPONSE" },
+		{ ISCSI_PDU_DATA_IN,
+			"DATA_IN" },
+		{ ISCSI_PDU_LOGOUT_RESPONSE,
+			"LOGOUT_RESPONSE" },
+		{ ISCSI_PDU_R2T,
+			"R2T" },
+		{ ISCSI_PDU_ASYNC_MSG,
+			"ASYNC_MSG" },
+		{ ISCSI_PDU_REJECT,
+			"REJECT" },
+		{ ISCSI_PDU_NO_PDU,
+			"NO_PDU" },
+		{0, NULL}
+	};
+
+	return iscsi_value_string_find(opcode_strings, opcode, "UNKNOWN");
+}
+
 void iscsi_dump_pdu_header(struct iscsi_context *iscsi, unsigned char *data) {
-	char dump[ISCSI_RAW_HEADER_SIZE*3+1]={0};
-	int i;
-	for (i=0;i<ISCSI_RAW_HEADER_SIZE;i++) {
-		snprintf(&dump[i * 3], 4, " %02x", data[i]);
+	char output[1024] = { 0 };
+	unsigned char iscsi_opcode;
+	const char *iscsi_opcode_string;
+	size_t output_off = 0;
+	int data_off = 0;
+
+
+	/* start to dump iSCSI opcode - data[0] */
+	iscsi_opcode = data[data_off];
+	iscsi_opcode_string = iscsi_opcode_str(iscsi_opcode);
+	output_off += snprintf(&output[output_off], sizeof(output) - output_off - 1, "%02x[%s]",
+			iscsi_opcode, iscsi_opcode_string);
+	data_off++;
+
+	if (iscsi_opcode == ISCSI_PDU_SCSI_REQUEST) {
+		unsigned char scsi_opcode;
+		const char *scsi_opcode_string;
+
+		/* the rest iSCSI PDU: data[1] - data[31] */
+		for ( ; data_off < 32; data_off++) {
+			if (sizeof(output) - 1 > output_off) {
+				output_off += snprintf(&output[output_off], sizeof(output) - output_off - 1,
+						" %02x", data[data_off]);
+			}
+		}
+
+		/* SCSI opcode: data[32] */
+		scsi_opcode = data[data_off];
+		scsi_opcode_string = scsi_opcode_str(scsi_opcode);
+		if (sizeof(output) - 1 > output_off) {
+			output_off += snprintf(&output[output_off], sizeof(output) - output_off - 1,
+					" %02x[%s]", scsi_opcode, scsi_opcode_string);
+			data_off++;
+		}
+
+		/* the rest SCSI PDU: data[33] - data[ISCSI_RAW_HEADER_SIZE - 1] */
+		for ( ; data_off < ISCSI_RAW_HEADER_SIZE; data_off++) {
+			if (sizeof(output) - 1 > output_off) {
+				output_off += snprintf(&output[output_off], sizeof(output) - output_off - 1,
+						" %02x", data[data_off]);
+			}
+		}
+	} else {
+		/* the rest iSCSI PDU: data[1] - data[ISCSI_RAW_HEADER_SIZE - 1] */
+		for ( ; data_off < ISCSI_RAW_HEADER_SIZE; data_off++) {
+			if (sizeof(output) - 1 > output_off) {
+				output_off += snprintf(&output[output_off], sizeof(output) - output_off - 1,
+						" %02x", data[data_off]);
+			}
+		}
 	}
-	ISCSI_LOG(iscsi, 2, "PDU header:%s", dump);
+
+	ISCSI_LOG(iscsi, 2, "PDU header: %s", output);
 }
 
 struct iscsi_pdu*
 iscsi_tcp_new_pdu(struct iscsi_context *iscsi, size_t size)
 {
-	struct iscsi_pdu *pdu;
-
-	pdu = iscsi_szmalloc(iscsi, size);
-
-	return pdu;
+	return iscsi_szmalloc(iscsi, size);
 }
 
 struct iscsi_pdu *
@@ -169,7 +259,7 @@ iscsi_tcp_free_pdu(struct iscsi_context *iscsi, struct iscsi_pdu *pdu)
 
 int
 iscsi_add_data(struct iscsi_context *iscsi, struct iscsi_data *data,
-	       unsigned char *dptr, int dsize, int pdualignment)
+	       const unsigned char *dptr, int dsize, int pdualignment)
 {
 	size_t len, aligned;
 
@@ -216,7 +306,7 @@ iscsi_add_data(struct iscsi_context *iscsi, struct iscsi_data *data,
 
 int
 iscsi_pdu_add_data(struct iscsi_context *iscsi, struct iscsi_pdu *pdu,
-		   unsigned char *dptr, int dsize)
+		   const unsigned char *dptr, int dsize)
 {
 	if (pdu == NULL) {
 		iscsi_set_error(iscsi, "trying to add data to NULL pdu");
@@ -330,7 +420,7 @@ int iscsi_process_target_nop_in(struct iscsi_context *iscsi,
 }
 
 static void iscsi_reconnect_after_logout(struct iscsi_context *iscsi, int status,
-                        void *command_data _U_, void *opaque _U_)
+                        void *command_data, void *opaque)
 {
 	if (status) {
 		ISCSI_LOG(iscsi, 1, "logout failed: %s", iscsi_get_error(iscsi));
@@ -622,7 +712,7 @@ iscsi_process_pdu(struct iscsi_context *iscsi, struct iscsi_in_pdu *in)
 			return -1;
 		}
 
-		if (is_finished) {
+		if (is_finished && iscsi->waitpdu != NULL) {
 			ISCSI_LIST_REMOVE(&iscsi->waitpdu, pdu);
 			iscsi->drv->free_pdu(iscsi, pdu);
 		}
@@ -713,16 +803,69 @@ iscsi_pdu_set_expxferlen(struct iscsi_pdu *pdu, uint32_t expxferlen)
 	scsi_set_uint32(&pdu->outdata.data[20], expxferlen);
 }
 
+/*
+ * A WRITE16 command[w] handles R2T, and queues DATAOUT PDU m,x,y,z:
+ *
+ *           outqueue->DATAOUT[x]->DATAOUT[y]->DATAOUT[z]...
+ *   outqueue_current->DATAOUT[m]
+ *          waitqueue->WRITE16[w]...
+ *
+ * 1, Once x, y, z gets released in initiator side, the target still expects the remaining
+ * DATAOUT PDUs.
+ * 2, Once command w timeout and callback to uplayer, uplayers usually releases memory of
+ *    iscsi task(include memory referenced by iovec.iov_base). DATAOUT[m] would access
+ *    invalid memory iovce.iov_base.
+ */
+static int iscsi_pdu_data_out_inprocess(struct iscsi_context *iscsi, struct iscsi_pdu *pdu)
+{
+	struct iscsi_pdu *tmp_pdu, *next_pdu;
+	enum scsi_opcode opcode = pdu->outdata.data[32];
+
+	/* only care DATA OUT command here */
+	if ((pdu->outdata.data[0] & 0x3f) != ISCSI_PDU_SCSI_REQUEST) {
+		return 0;
+	}
+
+	switch (opcode) {
+		case SCSI_OPCODE_WRITE10:
+		case SCSI_OPCODE_WRITE12:
+		case SCSI_OPCODE_WRITE16:
+			break;
+		default:
+			return 0;
+	};
+
+	/* current outgoing one is part of the PDU? */
+	if (iscsi->outqueue_current && (iscsi->outqueue_current->scsi_cbdata.task == pdu->scsi_cbdata.task)) {
+		return 1;
+	}
+
+	/* any child DATAOUT PDU in outqueue? */
+	for (tmp_pdu = iscsi->outqueue; tmp_pdu; tmp_pdu = next_pdu) {
+		next_pdu = tmp_pdu->next;
+
+		if (tmp_pdu->scsi_cbdata.task == pdu->scsi_cbdata.task) {
+			return 1;
+		}
+	}
+
+	return 0;
+}
+
 void
 iscsi_timeout_scan(struct iscsi_context *iscsi)
 {
 	struct iscsi_pdu *pdu;
 	struct iscsi_pdu *next_pdu;
 	time_t t = time(NULL);
+	uint32_t cmdsn_gap = 0;
 
 	for (pdu = iscsi->outqueue; pdu; pdu = next_pdu) {
 		next_pdu = pdu->next;
 
+                if (cmdsn_gap > 0) {
+			iscsi_pdu_set_cmdsn(pdu, pdu->cmdsn - cmdsn_gap);
+                }
 		if (pdu->scsi_timeout == 0) {
 			/* no timeout for this pdu */
 			continue;
@@ -731,8 +874,15 @@ iscsi_timeout_scan(struct iscsi_context *iscsi)
 			/* not expired yet */
 			continue;
 		}
+		if (!(pdu->outdata.data[0] & ISCSI_PDU_IMMEDIATE) &&
+		    (pdu->outdata.data[0] & 0x3f) != ISCSI_PDU_DATA_OUT) {
+			iscsi->cmdsn--;
+			cmdsn_gap++;
+		} else {
+			continue;
+		}
 		ISCSI_LIST_REMOVE(&iscsi->outqueue, pdu);
-		iscsi_set_error(iscsi, "command timed out");
+		iscsi_set_error(iscsi, "command timed out from outqueue");
 		iscsi_dump_pdu_header(iscsi, pdu->outdata.data);
 		if (pdu->callback) {
 			pdu->callback(iscsi, SCSI_STATUS_TIMEOUT,
@@ -751,8 +901,11 @@ iscsi_timeout_scan(struct iscsi_context *iscsi)
 			/* not expired yet */
 			continue;
 		}
+		if (iscsi_pdu_data_out_inprocess(iscsi, pdu)) {
+			continue;
+		}
 		ISCSI_LIST_REMOVE(&iscsi->waitpdu, pdu);
-		iscsi_set_error(iscsi, "command timed out");
+		iscsi_set_error(iscsi, "command timed out from waitqueue");
 		iscsi_dump_pdu_header(iscsi, pdu->outdata.data);
 		if (pdu->callback) {
 			pdu->callback(iscsi, SCSI_STATUS_TIMEOUT,
@@ -775,21 +928,19 @@ iscsi_cancel_pdus(struct iscsi_context *iscsi)
 
 	while ((pdu = iscsi->outqueue)) {
 		ISCSI_LIST_REMOVE(&iscsi->outqueue, pdu);
-		if (iscsi->is_loggedin && pdu->callback) {
-			/* If an error happened during connect/login,
-			   we don't want to call any of the callbacks.
-			*/
+		if (pdu->callback) {
 			pdu->callback(iscsi, SCSI_STATUS_CANCELLED,
 			              NULL, pdu->private_data);
 		}
+		if (!(pdu->outdata.data[0] & ISCSI_PDU_IMMEDIATE) &&
+		    (pdu->outdata.data[0] & 0x3f) != ISCSI_PDU_DATA_OUT) {
+			iscsi->cmdsn--;
+		}
 		iscsi->drv->free_pdu(iscsi, pdu);
 	}
 	while ((pdu = iscsi->waitpdu)) {
 		ISCSI_LIST_REMOVE(&iscsi->waitpdu, pdu);
-		if (iscsi->is_loggedin && pdu->callback) {
-			/* If an error happened during connect/login,
-			   we don't want to call any of the callbacks.
-			*/
+		if (pdu->callback) {
 			pdu->callback(iscsi, SCSI_STATUS_CANCELLED,
 			              NULL, pdu->private_data);
 		}
