@@ -105,11 +105,19 @@ iscsi_add_to_outqueue(struct iscsi_context *iscsi, struct iscsi_pdu *pdu)
 	/* queue pdus in ascending order of CmdSN. 
 	 * ensure that pakets with the same CmdSN are kept in FIFO order.
 	 * immediate PDUs are queued in front of queue with the CmdSN
-	 * of the first element in the outqueue.
+	 * of the first cmd pdu in the outqueue.
 	 */
 
 	if (pdu->outdata.data[0] & ISCSI_PDU_IMMEDIATE) {
-		iscsi_pdu_set_cmdsn(pdu, current->cmdsn);
+		do {
+			if ((current->outdata.data[0] & 0x3f) != ISCSI_PDU_DATA_OUT) {
+				iscsi_pdu_set_cmdsn(pdu, current->cmdsn);
+				break;
+			}
+			current = current->next;
+		} while (current != NULL);
+
+		current = iscsi->outqueue;
 	}
 
 	do {
@@ -220,7 +228,7 @@ static int iscsi_tcp_connect(struct iscsi_context *iscsi, union socket_address *
 	}
 
 	if (iscsi->old_iscsi && iscsi->fd != iscsi->old_iscsi->fd) {
-		if (dup2(iscsi->fd, iscsi->old_iscsi->fd) == -1) {
+		if (iscsi_dup2(iscsi, iscsi->fd, iscsi->old_iscsi->fd) == -1) {
 			return -1;
 		}
 		close(iscsi->fd);
@@ -425,6 +433,9 @@ iscsi_tcp_disconnect(struct iscsi_context *iscsi)
 int
 iscsi_disconnect(struct iscsi_context *iscsi)
 {
+	if (!iscsi || !iscsi->drv || !iscsi->drv->disconnect)
+		return -1;
+
         return iscsi->drv->disconnect(iscsi);
 }
 
@@ -505,20 +516,22 @@ iscsi_out_queue_length(struct iscsi_context *iscsi)
 ssize_t
 iscsi_iovector_readv_writev(struct iscsi_context *iscsi, struct scsi_iovector *iovector, uint32_t pos, ssize_t count, int do_write)
 {
-        struct scsi_iovec *iov, *iov2;
-        int niov;
-        uint32_t len2;
-        size_t _len2;
-        ssize_t n;
+	struct scsi_iovec *iov, *iov2;
+	int niov;
+	uint32_t len2;
+	size_t _len2;
+	ssize_t n;
+	const char *rw = do_write ? "write" : "read";
 
-        if (iovector->iov == NULL) {
+	if (iovector->iov == NULL) {
+		iscsi_set_error(iscsi, "%s: iovector empty buffer", rw);
 		errno = EINVAL;
 		return -1;
 	}
 
 	if (pos < iovector->offset) {
-		iscsi_set_error(iscsi, "iovector reset. pos is smaller than"
-				"current offset");
+		iscsi_set_error(iscsi, "%s: iovector reset. pos(%d) is smaller than"
+				"current offset(%ld)", rw, pos, iovector->offset);
 		errno = EINVAL;
 		return -1;
 	}
@@ -527,6 +540,8 @@ iscsi_iovector_readv_writev(struct iscsi_context *iscsi, struct scsi_iovector *i
 		/* someone issued a read/write but did not provide enough user buffers for all the data.
 		 * maybe someone tried to read just 512 bytes off a MMC device?
 		 */
+		iscsi_set_error(iscsi, "%s: iovector consumed(%d) exceeds niov(%d)",
+				rw, iovector->consumed, iovector->niov);
 		errno = EINVAL;
 		return -1;
 	}
@@ -541,6 +556,8 @@ iscsi_iovector_readv_writev(struct iscsi_context *iscsi, struct scsi_iovector *i
 		iovector->consumed++;
 		pos -= iov->iov_len;
 		if (iovector->niov <= iovector->consumed) {
+			iscsi_set_error(iscsi, "%s: iovector consumed(%d) exceeds niov(%d) on head",
+					rw, iovector->consumed, iovector->niov);
 			errno = EINVAL;
 			return -1;
 		}
@@ -556,6 +573,9 @@ iscsi_iovector_readv_writev(struct iscsi_context *iscsi, struct scsi_iovector *i
 	while (len2 > iov2->iov_len) {
 		niov++;
 		if (iovector->niov < iovector->consumed + niov) {
+			iscsi_set_error(iscsi, "%s: iovector consumed(%d) + niov(%d)"
+					" exceeds niov(%d) on tail",
+					rw, iovector->consumed, niov, iovector->niov);
 			errno = EINVAL;
 			return -1;
 		}
@@ -588,6 +608,9 @@ iscsi_iovector_readv_writev(struct iscsi_context *iscsi, struct scsi_iovector *i
 		errno = EINVAL;
 		return -1;
 	}
+	if ((n <= 0) && (errno != EAGAIN) && (errno == EINTR)) {
+		iscsi_set_error(iscsi, "%s: socket failed, errno:%d", rw, errno);
+	}
 	return n;
 }
 
@@ -619,7 +642,8 @@ iscsi_read_from_socket(struct iscsi_context *iscsi)
 			 * no need to limit the read to what is available in the socket
 			 */
 			count = hdr_size - in->hdr_pos;
-			count = recv(iscsi->fd, &in->hdr[in->hdr_pos], count, 0);
+			count = recv(iscsi->fd, (void *)&in->hdr[in->hdr_pos],
+                                     count, 0);
 			if (count == 0) {
 				/* remote side has closed the socket. */
 				return -1;
@@ -670,7 +694,7 @@ iscsi_read_from_socket(struct iscsi_context *iscsi)
 					}
 					buf = &in->data[in->data_pos];
 				}
-				count = recv(iscsi->fd, buf, count, 0);
+				count = recv(iscsi->fd, (void *)buf, count, 0);
 			}
 			if (count == 0) {
 				/* remote side has closed the socket. */
@@ -680,9 +704,6 @@ iscsi_read_from_socket(struct iscsi_context *iscsi)
 				if (errno == EINTR || errno == EAGAIN) {
 					break;
 				}
-				iscsi_set_error(iscsi, "read from socket failed, "
-						"errno:%d %s", errno,
-						iscsi_get_error(iscsi));
 				return -1;
 			}
 			in->data_pos += count;
@@ -794,7 +815,8 @@ iscsi_write_to_socket(struct iscsi_context *iscsi)
 		/* Write header and any immediate data */
 		if (pdu->outdata_written < pdu->outdata.size) {
 			count = send(iscsi->fd,
-				     pdu->outdata.data + pdu->outdata_written,
+				     (void *)(pdu->outdata.data +
+                                              pdu->outdata_written),
 				     pdu->outdata.size - pdu->outdata_written,
 				     socket_flags);
 			if (count == -1) {
@@ -831,9 +853,6 @@ iscsi_write_to_socket(struct iscsi_context *iscsi)
 				if (errno == EAGAIN || errno == EWOULDBLOCK) {
 					return 0;
 				}
-				iscsi_set_error(iscsi, "Error when writing to "
-						"socket :%d %s", errno,
-						iscsi_get_error(iscsi));
 				return -1;
 			}
 
@@ -902,7 +921,7 @@ iscsi_tcp_service(struct iscsi_context *iscsi, int revents)
 			return iscsi_reconnect(iscsi);
 		} else {
 			if (iscsi->old_iscsi) {
-				return 0;
+				goto check_timeout;
 			}
 		}
 	}
@@ -980,16 +999,24 @@ iscsi_tcp_service(struct iscsi_context *iscsi, int revents)
 
 	if (revents & POLLIN) {
 		if (iscsi_read_from_socket(iscsi) != 0) {
+			ISCSI_LOG(iscsi, 1, "%s", iscsi_get_error(iscsi));
 			return iscsi_service_reconnect_if_loggedin(iscsi);
 		}
 	}
 	if (revents & POLLOUT) {
 		if (iscsi_write_to_socket(iscsi) != 0) {
+			ISCSI_LOG(iscsi, 1, "%s", iscsi_get_error(iscsi));
 			return iscsi_service_reconnect_if_loggedin(iscsi);
 		}
 	}
+
+check_timeout:
 	iscsi_timeout_scan(iscsi);
 
+	if (iscsi->old_iscsi) {
+		iscsi_timeout_scan(iscsi->old_iscsi);
+	}
+
 	return 0;
 }
 
@@ -1052,7 +1079,7 @@ void iscsi_set_tcp_keepintvl(struct iscsi_context *iscsi, int value)
 	ISCSI_LOG(iscsi, 2, "TCP_KEEPINTVL will be set to %d on next socket creation",value);
 }
 
-int iscsi_set_tcp_keepalive(struct iscsi_context *iscsi, int idle _U_, int count _U_, int interval _U_)
+int iscsi_set_tcp_keepalive(struct iscsi_context *iscsi, int idle, int count, int interval)
 {
 #ifdef SO_KEEPALIVE
 	int value = 1;
@@ -1087,7 +1114,7 @@ int iscsi_set_tcp_keepalive(struct iscsi_context *iscsi, int idle _U_, int count
 	return 0;
 }
 
-void iscsi_set_bind_interfaces(struct iscsi_context *iscsi, char * interfaces _U_)
+void iscsi_set_bind_interfaces(struct iscsi_context *iscsi, char * interfaces)
 {
 #if __linux
 	strncpy(iscsi->bind_interfaces,interfaces,MAX_STRING_SIZE);
