@@ -54,11 +54,27 @@ static void
 event_loop(struct iscsi_context *iscsi, struct iscsi_sync_state *state)
 {
         struct pollfd pfd;
+	int scsi_timeout;
 	int ret;
+	time_t t;
 
+        if (iscsi->scsi_timeout) {
+		scsi_timeout = time(NULL) + iscsi->scsi_timeout;
+	} else {
+		scsi_timeout = 0;
+	}
+                
 	while (state->finished == 0) {
 		short revents;
 
+                if (scsi_timeout) {
+                        t = time(NULL);
+                        if (t > scsi_timeout) {
+                                iscsi_set_error(iscsi, "Connect timedout");
+                                state->status = -1;
+                                return;
+                        }
+                }
 		pfd.fd = iscsi_get_fd(iscsi);
 		pfd.events = iscsi_which_events(iscsi);
 
@@ -82,8 +98,8 @@ event_loop(struct iscsi_context *iscsi, struct iscsi_sync_state *state)
  * Synchronous iSCSI commands
  */
 static void
-iscsi_sync_cb(struct iscsi_context *iscsi _U_, int status,
-	      void *command_data _U_, void *private_data)
+iscsi_sync_cb(struct iscsi_context *iscsi, int status,
+	      void *command_data, void *private_data)
 {
 	struct iscsi_sync_state *state = private_data;
 
@@ -226,6 +242,22 @@ int iscsi_reconnect_sync(struct iscsi_context *iscsi)
 	return (state.status == SCSI_STATUS_GOOD) ? 0 : -1;
 }
 
+int iscsi_force_reconnect_sync(struct iscsi_context *iscsi)
+{
+	struct iscsi_sync_state state;
+
+	memset(&state, 0, sizeof(state));
+
+	if (iscsi_force_reconnect(iscsi) != 0) {
+		iscsi_set_error(iscsi, "Failed to reconnect. %s", iscsi_get_error(iscsi));
+		return -1;
+	}
+
+	reconnect_event_loop(iscsi, &state);
+
+	return (state.status == SCSI_STATUS_GOOD) ? 0 : -1;
+}
+
 static void
 iscsi_task_mgmt_sync_cb(struct iscsi_context *iscsi, int status,
 	      void *command_data, void *private_data)
@@ -340,7 +372,7 @@ iscsi_task_mgmt_target_cold_reset_sync(struct iscsi_context *iscsi)
  * Synchronous SCSI commands
  */
 static void
-scsi_sync_cb(struct iscsi_context *iscsi _U_, int status, void *command_data,
+scsi_sync_cb(struct iscsi_context *iscsi, int status, void *command_data,
 	     void *private_data)
 {
 	struct scsi_task *task = command_data;
@@ -1820,7 +1852,7 @@ iscsi_modesense10_sync(struct iscsi_context *iscsi, int lun, int llbaa, int dbd,
 	return state.task;
 }
 
-void iscsi_free_discovery_data(struct iscsi_context *iscsi _U_,
+void iscsi_free_discovery_data(struct iscsi_context *iscsi,
                                struct iscsi_discovery_address *da)
 {
         while (da) {
@@ -1828,18 +1860,19 @@ void iscsi_free_discovery_data(struct iscsi_context *iscsi _U_,
 
                 while (da->portals) {
                         struct iscsi_target_portal *ponext = da->portals->next;
-                        free(discard_const(da->portals->portal));
+
+                        free(da->portals->portal);
                         free(da->portals);
                         da->portals = ponext;
                 }
-                free(discard_const(da->target_name));
+                free(da->target_name);
                 free(da);
                 da = danext;
         }
 }
 
 static void
-iscsi_discovery_cb(struct iscsi_context *iscsi _U_, int status,
+iscsi_discovery_cb(struct iscsi_context *iscsi, int status,
 	      void *command_data, void *private_data)
 {
 	struct iscsi_sync_state *state = private_data;
