@@ -27,9 +27,8 @@
 #include <unistd.h>
 #include <stdlib.h>
 
-#ifndef discard_const
-#define discard_const(ptr) ((void *)((intptr_t)(ptr)))
-#endif
+#include "iscsi.h"
+#include "scsi-lowlevel.h"
 
 extern const char *initiatorname1;
 extern const char *initiatorname2;
@@ -50,23 +49,24 @@ extern const char *initiatorname2;
 #define EXPECT_RESERVATION_CONFLICT SCSI_STATUS_RESERVATION_CONFLICT, 0, NULL, 0
 #define EXPECT_COPY_ABORTED SCSI_STATUS_CHECK_CONDITION, SCSI_SENSE_COPY_ABORTED, copy_aborted_ascqs, 3
 
-int no_medium_ascqs[3];
-int lba_oob_ascqs[1];
-int invalid_cdb_ascqs[2];
-int param_list_len_err_ascqs[1];
-int too_many_desc_ascqs[2];
-int unsupp_desc_code_ascqs[2];
-int write_protect_ascqs[3];
-int sanitize_ascqs[1];
-int removal_ascqs[1];
-int miscompare_ascqs[1];
-int copy_aborted_ascqs[3];
+extern int no_medium_ascqs[3];
+extern int lba_oob_ascqs[1];
+extern int invalid_cdb_ascqs[2];
+extern int param_list_len_err_ascqs[1];
+extern int too_many_desc_ascqs[2];
+extern int unsupp_desc_code_ascqs[2];
+extern int write_protect_ascqs[3];
+extern int sanitize_ascqs[1];
+extern int removal_ascqs[1];
+extern int miscompare_ascqs[1];
+extern int copy_aborted_ascqs[3];
 
 extern int loglevel;
 #define LOG_SILENT  0
 #define LOG_NORMAL  1
 #define LOG_VERBOSE 2
-void logging(int level, const char *format, ...) _R_(2,3);
+void logging(int level, const char *format, ...)
+__attribute__((format(printf, 2, 3)));
 
 /*
  * define special flags for logging a blank line, so compiler
@@ -183,6 +183,16 @@ do {                                                                        \
         }                                                                \
 } while (0);
 
+#define CHECK_FOR_ISCSI(_sd)                                            \
+do {                                                                    \
+        if (_sd->iscsi_ctx == NULL) {                                   \
+                logging(LOG_NORMAL, "[SKIPPED] Not an iSCSI device."    \
+                        " Skipping test");                              \
+                CU_PASS("[SKIPPED] Not an iSCSI device. Skipping test"); \
+                return;                                                 \
+        }                                                               \
+} while (0);
+
 #define CHECK_SIZE(_min_blocks, _c)                                     \
         do {                                                            \
                 if (num_blocks < _min_blocks) {                         \
@@ -779,7 +789,6 @@ struct iscsi_async_state {
 void wait_until_test_finished(struct iscsi_context *iscsi, struct iscsi_async_state *test_state);
 
 struct iscsi_pdu;
-int (*local_iscsi_queue_pdu)(struct iscsi_context *iscsi, struct iscsi_pdu *pdu);
 
 struct scsi_command_descriptor *get_command_descriptor(int opcode, int sa);
 
@@ -906,9 +915,16 @@ int clear_swp(struct scsi_device *sdev);
 
 int extendedcopy(struct scsi_device *sdev, struct iscsi_data *data, int status, enum scsi_sense_key key, int *ascq, int num_ascq);
 int get_desc_len(enum ec_descr_type_code desc_type);
-int populate_tgt_desc(unsigned char *desc, enum ec_descr_type_code desc_type, int luid_type, int nul, int peripheral_type, int rel_init_port_id, int pad, struct scsi_device *dev);
-int populate_seg_desc_hdr(unsigned char *hdr, enum ec_descr_type_code desc_type, int dc, int cat, int src_index, int dst_index);
-int populate_seg_desc_b2b(unsigned char *desc, int dc, int cat, int src_index, int dst_index, int num_blks, uint64_t src_lba, uint64_t dst_lba);
+int populate_tgt_desc(uint8_t desc[32], enum ec_descr_type_code desc_type,
+                      int luid_type, int nul, int peripheral_type,
+                      uint16_t rel_init_port_id, int pad,
+                      struct scsi_device *dev);
+int populate_seg_desc_hdr(unsigned char *hdr, enum ec_descr_type_code desc_type, int dc, int cat, uint16_t src_index, uint16_t dst_index);
+int populate_seg_desc_b2b(unsigned char *desc, int dc, int cat, int src_index, int dst_index, uint16_t num_blks, uint64_t src_lba, uint64_t dst_lba);
+int populate_seg_desc_b2b_off(unsigned char *desc, int cat, int src_index,
+                              int dst_index, uint32_t num_bytes,
+                              uint64_t src_lba, uint64_t dst_lba,
+                              uint16_t src_byte_off, uint16_t dst_byte_off);
 void populate_param_header(unsigned char *buf, int list_id, int str, int list_id_usage, int prio, int tgt_desc_len, int seg_desc_len, int inline_data_len);
 int receive_copy_results(struct scsi_task **task, struct scsi_device *sdev,
                          enum scsi_copy_results_sa sa, int list_id,
