@@ -29,11 +29,14 @@
 #include "config.h"
 #endif
 
+#include "iscsi.h"
 #include "iscsi-support.h"
 
 /* globals between setup, tests, and teardown */
 extern struct scsi_task *task;
 extern unsigned char *read_write_buf;
+extern int (*orig_queue_pdu)(struct iscsi_context *iscsi,
+                             struct iscsi_pdu *pdu);
 
 #ifndef HAVE_CU_SUITEINFO_PSETUPFUNC
 /* libcunit version 1 */
@@ -51,6 +54,7 @@ void test_teardown(void);
 void test_compareandwrite_simple(void);
 void test_compareandwrite_dpofua(void);
 void test_compareandwrite_miscompare(void);
+void test_compareandwrite_miscompare_sense(void);
 void test_compareandwrite_unwritten(void);
 void test_compareandwrite_invalid_dataout_size(void);
 
@@ -60,6 +64,7 @@ void test_extendedcopy_descr_limits(void);
 void test_extendedcopy_descr_type(void);
 void test_extendedcopy_validate_tgt_descr(void);
 void test_extendedcopy_validate_seg_descr(void);
+void test_extendedcopy_large(void);
 
 void test_get_lba_status_simple(void);
 void test_get_lba_status_beyond_eol(void);
@@ -77,6 +82,11 @@ void test_iscsi_cmdsn_toohigh(void);
 void test_iscsi_cmdsn_toolow(void);
 
 void test_iscsi_datasn_invalid(void);
+void test_iscsi_sendtargets_simple(void);
+void test_iscsi_sendtargets_invalid(void);
+void test_iscsi_nop_simple(void);
+void test_iscsi_chap_simple(void);
+void test_iscsi_chap_invalid(void);
 
 void test_mandatory_sbc(void);
 
@@ -180,6 +190,8 @@ void test_readonly_sbc(void);
 void test_receive_copy_results_copy_status(void);
 void test_receive_copy_results_op_params(void);
 
+void test_report_luns_simple(void);
+
 void test_report_supported_opcodes_one_command(void);
 void test_report_supported_opcodes_rctd(void);
 void test_report_supported_opcodes_servactv(void);
@@ -249,6 +261,7 @@ void test_write10_wrprotect(void);
 void test_write10_dpofua(void);
 void test_write10_residuals(void);
 void test_async_write(void);
+void test_async_io_logout(void);
 
 void test_write12_simple(void);
 void test_write12_beyond_eol(void);
