@@ -66,6 +66,7 @@ test_async_reset_cb(struct iscsi_context *iscsi __attribute__((unused)),
 
 	/* command_data NULL if a reconnect occured. see iscsi_reconnect_cb() */
 	reconnect_succeeded = command_data != NULL;
+	CU_ASSERT_EQUAL(reconnect_succeeded, 1);
 	if (!reconnect_succeeded)
 		return;
 	tmf_response = *(uint32_t *)command_data;
@@ -91,17 +92,13 @@ test_async_lu_reset_simple(void)
 {
 	int ret;
 	struct tests_async_reset_state state = { NULL, 0, 0, 0, };
-	int blocksize = 512;
 	int blocks_per_io = 8;
-	unsigned char buf[blocksize * blocks_per_io];
+	unsigned char *buf;
 	uint64_t timeout_sec;
 
 	CHECK_FOR_DATALOSS;
 	CHECK_FOR_SBC;
-	if (sd->iscsi_ctx == NULL) {
-                CU_PASS("[SKIPPED] Non-iSCSI");
-		return;
-	}
+	CHECK_FOR_ISCSI(sd);
 
 	if (maximum_transfer_length
 	 && (maximum_transfer_length < (int)(blocks_per_io))) {
@@ -109,15 +106,18 @@ test_async_lu_reset_simple(void)
 		return;
 	}
 
-	memset(buf, 0, blocksize * blocks_per_io);
+	buf = calloc(block_size, blocks_per_io);
+	CU_ASSERT(buf != NULL);
+	if (!buf)
+		return;
 
 	/* queue and dispatch write before the reset */
-	state.wtask = scsi_cdb_write10(0, blocks_per_io * blocksize,
-				 blocksize, 0, 0, 0, 0, 0);
+	state.wtask = scsi_cdb_write10(0, blocks_per_io * block_size,
+				 block_size, 0, 0, 0, 0, 0);
 	CU_ASSERT_PTR_NOT_NULL_FATAL(state.wtask);
 
 	ret = scsi_task_add_data_out_buffer(state.wtask,
-					    blocks_per_io * blocksize,
+					    blocks_per_io * block_size,
 					    buf);
 	CU_ASSERT_EQUAL(ret, 0);
 
@@ -154,9 +154,6 @@ test_async_lu_reset_simple(void)
 				    0xffffffff, 0,
 				    test_async_reset_cb, &state);
 	CU_ASSERT_EQUAL(ret, 0);
-	CU_ASSERT_EQUAL(reconnect_succeeded, 1);
-	if (!reconnect_succeeded)
-		goto out;
 
 	logging(LOG_VERBOSE, "LU RESET queued");
 
@@ -198,11 +195,11 @@ test_async_lu_reset_simple(void)
 		CU_FAIL("unexpected WRITE/RESET state");
 	}
 
-out:
-	scsi_free_scsi_task(state.wtask);
-
 	/* Avoid that callbacks get invoked after this test finished */
-        iscsi_logout_sync(sd->iscsi_ctx);
-        iscsi_destroy_context(sd->iscsi_ctx);
+	iscsi_logout_sync(sd->iscsi_ctx);
+	iscsi_destroy_context(sd->iscsi_ctx);
+	scsi_free_scsi_task(state.wtask);
 	sd->iscsi_ctx = NULL;
+
+	free(buf);
 }
