@@ -30,41 +30,41 @@ static int my_iscsi_queue_pdu(struct iscsi_context *iscsi, struct iscsi_pdu *pdu
 {
         switch (change_cmdsn) {
         case 1:
-                /* change the cmdsn so it becomes too big */
+                /*
+		 * change the cmdsn so it becomes too big. TODO: use
+		 * iscsi_pdu_set_cmdsn(), which also changes pdu->cmdsn?
+		 */
                 scsi_set_uint32(&pdu->outdata.data[24], iscsi->maxcmdsn + 1);
-                /* fudge the cmdsn value back to where it should be if this
-                 * pdu is ignored.
+                /*
+                 * Fudge the cmdsn value back to where it should be if this pdu
+                 * is ignored. The cmdsn value will be incremented by the
+                 * orig_queue_pdu(), so it's off by one.
                  */
-                iscsi->cmdsn = iscsi->expcmdsn;
+                iscsi->cmdsn = iscsi->expcmdsn - 1;
                 break;
         }
 
-        change_cmdsn = 0;        
-        return 0;
+        change_cmdsn = 0;
+        return orig_queue_pdu(iscsi, pdu);
 }
 
 void test_iscsi_cmdsn_toohigh(void)
-{ 
+{
         int ret;
 
         logging(LOG_VERBOSE, LOG_BLANK_LINE);
         logging(LOG_VERBOSE, "Test sending invalid iSCSI CMDSN");
         logging(LOG_VERBOSE, "CMDSN MUST be in the range EXPCMDSN and MAXCMDSN");
 
-        if (sd->iscsi_ctx == NULL) {
-                const char *err = "[SKIPPED] This test is "
-                        "only supported for iSCSI backends";
-                logging(LOG_NORMAL, "%s", err);
-                CU_PASS(err);
-                return;
-        }
+        CHECK_FOR_ISCSI(sd);
 
         logging(LOG_VERBOSE, "RFC3720:3.2.2.1 CMDSN > MAXCMDSN must be silently ignored by the target");
         logging(LOG_VERBOSE, "Send a TESTUNITREADY with CMDSN == MAXCMDSN+1. Should be ignored by the target.");
 
         sd->iscsi_ctx->use_immediate_data = ISCSI_IMMEDIATE_DATA_NO;
         sd->iscsi_ctx->target_max_recv_data_segment_length = block_size;
-        local_iscsi_queue_pdu = my_iscsi_queue_pdu;
+        /* override transport queue_pdu callback for PDU manipulation */
+        sd->iscsi_ctx->drv->queue_pdu = my_iscsi_queue_pdu;
         change_cmdsn = 1;
         /* we don't want autoreconnect since some targets will incorrectly
          * drop the connection on this condition.
@@ -81,7 +81,6 @@ void test_iscsi_cmdsn_toohigh(void)
                 logging(LOG_VERBOSE, "[FAILURE] We got a response from the target but SMDSN was outside of the window.");
         }
 
-        
         iscsi_set_noautoreconnect(sd->iscsi_ctx, 0);
         logging(LOG_VERBOSE, "Send a TESTUNITREADY with CMDSN == EXPCMDSN. should work again");
         TESTUNITREADY(sd,
