@@ -1,4 +1,3 @@
-/* -*-  mode:c; tab-width:8; c-basic-offset:8; indent-tabs-mode:nil;  -*- */
 /* 
    Copyright (C) 2013 by Lee Duncan <lee@gonzoleeman.net>
    
@@ -38,12 +37,11 @@ test_prin_serviceaction_range(void)
         logging(LOG_VERBOSE, "Test Persistent Reserve IN Serviceaction range.");
 
         /* verify PRIN/READ_KEYS works -- XXX redundant -- remove this? */
-        ret = prin_read_keys(sd, &task, NULL);
+        ret = prin_read_keys(sd, &task, NULL, 16384);
         if (ret == -2) {
-                logging(LOG_NORMAL, "[SKIPPED] PERSISTEN RESERVE IN is not implemented.");
                 CU_PASS("PERSISTENT RESERVE IN is not implemented.");
                 return;
-        }        
+        }
         CU_ASSERT_EQUAL(ret, 0);
 
         /* verify that PRIN/SA={0,1,2,3} works ... */
