@@ -1,4 +1,3 @@
-/* -*-  mode:c; tab-width:8; c-basic-offset:8; indent-tabs-mode:nil;  -*- */
 /* 
    Copyright (C) 2013 by Lee Duncan <lee@gonzoleeman.net>
    
@@ -40,7 +39,6 @@ test_prout_register_simple(void)
         /* register our reservation key with the target */
         ret = prout_register_and_ignore(sd, key);
         if (ret == -2) {
-                logging(LOG_NORMAL, "[SKIPPED] PERSISTEN RESERVE OUT is not implemented.");
                 CU_PASS("PERSISTENT RESERVE OUT is not implemented.");
                 return;
         }        
