@@ -1,4 +1,3 @@
-/* -*-  mode:c; tab-width:8; c-basic-offset:8; indent-tabs-mode:nil;  -*- */
 /* 
    Copyright (C) 2013 Ronnie Sahlberg <ronniesahlberg@gmail.com>
    
@@ -46,7 +45,7 @@ test_startstopunit_pwrcnd(void)
 
                 logging(LOG_VERBOSE, "Test TESTUNITREADY that medium is not ejected.");
                 TESTUNITREADY(sd,
-                              EXPECT_SANITIZE);
+                              EXPECT_STATUS_GOOD);
         }
 
         logging(LOG_VERBOSE, "In case the target did eject the medium, load it again.");
