@@ -1,4 +1,3 @@
-/* -*-  mode:c; tab-width:8; c-basic-offset:8; indent-tabs-mode:nil;  -*- */
 /* 
    Copyright (C) 2013 Ronnie Sahlberg <ronniesahlberg@gmail.com>
    
@@ -37,9 +36,12 @@ test_writesame10_unmap_until_end(void)
         CHECK_FOR_SBC;
 
         if (inq_bl->wsnz) {
-            logging(LOG_NORMAL, "[SKIPPED] WRITESAME10 does not support 0-blocks.");
-            CU_PASS("[SKIPPED] WRITESAME10 does not support 0-blocks.");
-            return;
+                logging(LOG_NORMAL, "WRITESAME10 does not support 0-blocks."
+                        "WSNZ == 1");
+                memset(scratch, 0, block_size);
+                WRITESAME10(sd, 0, block_size, 0, 0, 1, 0, 0, scratch,
+                            EXPECT_INVALID_FIELD_IN_CDB);
+                return;
         }
 
         logging(LOG_VERBOSE, LOG_BLANK_LINE);
