@@ -20,6 +20,8 @@
 
 #ifdef HAVE_POLL_H
 #include <poll.h>
+#else
+#include "win32_compat.h"
 #endif
 
 #ifdef HAVE_UNISTD_H
@@ -37,10 +39,6 @@
 #include "iscsi-private.h"
 #include "scsi-lowlevel.h"
 
-#ifndef discard_const
-#define discard_const(ptr) ((void *)((intptr_t)(ptr)))
-#endif
-
 const char *initiator = "iqn.2007-10.com.github:sahlberg:libiscsi:prog-noop-reply";
 
 struct client_state {
@@ -86,8 +84,8 @@ void event_loop(struct iscsi_context *iscsi, struct client_state *state,
 	}
 }
 
-void tur_cb(struct iscsi_context *iscsi _U_, int status,
-	    void *command_data _U_, void *private_data)
+void tur_cb(struct iscsi_context *iscsi, int status,
+	    void *command_data, void *private_data)
 {
 	struct client_state *state = (struct client_state *)private_data;
 
@@ -136,7 +134,7 @@ int main(int argc, char *argv[])
 	struct iscsi_context *iscsi;
 	struct iscsi_url *iscsi_url = NULL;
 	struct client_state state;
-	const char *url = NULL;
+	char *url = NULL;
 	int c;
 	static int show_help = 0, show_usage = 0, debug = 0;
 
@@ -210,10 +208,8 @@ int main(int argc, char *argv[])
 	}
 
 	iscsi_url = iscsi_parse_full_url(iscsi, url);
-	
-	if (url) {
-		free(discard_const(url));
-	}
+
+        free(url);
 
 	if (iscsi_url == NULL) {
 		fprintf(stderr, "Failed to parse URL: %s\n", 
