@@ -31,10 +31,6 @@
 #include "iscsi.h"
 #include "scsi-lowlevel.h"
 
-#ifndef discard_const
-#define discard_const(ptr) ((void *)((intptr_t)(ptr)))
-#endif
-
 const char *initiator = "iqn.2007-10.com.github:sahlberg:libiscsi:iscsi-inq";
 
 void inquiry_block_limits(struct scsi_inquiry_block_limits *inq)
@@ -236,7 +232,7 @@ void print_help(void)
 int main(int argc, char *argv[])
 {
 	struct iscsi_context *iscsi;
-	const char *url = NULL;
+	char *url = NULL;
 	struct iscsi_url *iscsi_url = NULL;
 	int evpd = 0, pagecode = 0;
 	int show_help = 0, show_usage = 0, debug = 0;
@@ -245,7 +241,7 @@ int main(int argc, char *argv[])
 	static struct option long_options[] = {
 		{"help",           no_argument,          NULL,        'h'},
 		{"usage",          no_argument,          NULL,        'u'},
-		{"debug",          no_argument,          NULL,        'd'},
+		{"debug",          required_argument,    NULL,        'd'},
 		{"initiator-name", required_argument,    NULL,        'i'},
 		{"evpd",           required_argument,    NULL,        'e'},
 		{"pagecode",       required_argument,    NULL,        'c'},
@@ -253,7 +249,7 @@ int main(int argc, char *argv[])
 	};
 	int option_index;
 
-	while ((c = getopt_long(argc, argv, "h?udi:e:c:", long_options,
+	while ((c = getopt_long(argc, argv, "h?ud:i:e:c:", long_options,
 			&option_index)) != -1) {
 		switch (c) {
 		case 'h':
@@ -264,16 +260,16 @@ int main(int argc, char *argv[])
 			show_usage = 1;
 			break;
 		case 'd':
-			debug = 1;
+			debug = strtol(optarg, NULL, 0);
 			break;
 		case 'i':
 			initiator = optarg;
 			break;
 		case 'e':
-			evpd = atoi(optarg);
+			evpd = strtol(optarg, NULL, 0);
 			break;
 		case 'c':
-			pagecode = atoi(optarg);
+			pagecode = strtol(optarg, NULL, 0);
 			break;
 		default:
 			fprintf(stderr, "Unrecognized option '%c'\n\n", c);
@@ -312,10 +308,8 @@ int main(int argc, char *argv[])
 		exit(10);
 	}
 	iscsi_url = iscsi_parse_full_url(iscsi, url);
-	
-	if (url) {
-		free(discard_const(url));
-	}
+
+        free(url);
 
 	if (iscsi_url == NULL) {
 		fprintf(stderr, "Failed to parse URL: %s\n", 
