@@ -21,7 +21,9 @@
 #if defined(_WIN32)
 #include <winsock2.h>
 #include "win32_compat.h"
+#ifdef _MSC_VER
 #pragma comment(lib, "ws2_32.lib")
+#endif
 WSADATA wsaData;
 #endif
 
@@ -37,13 +39,11 @@ WSADATA wsaData;
 #include <stdlib.h>
 #include <stdint.h>
 #include <string.h>
+#include <strings.h>
+#include <getopt.h>
 #include "iscsi.h"
 #include "scsi-lowlevel.h"
 
-#ifndef discard_const
-#define discard_const(ptr) ((void *)((intptr_t)(ptr)))
-#endif
-
 int showluns;
 int useurls;
 const char *initiator = "iqn.2007-10.com.github:sahlberg:libiscsi:iscsi-ls";
@@ -237,7 +237,7 @@ void list_luns(struct client_state *clnt, const char *target, const char *portal
 
 
 
-void discoverylogout_cb(struct iscsi_context *iscsi, int status, void *command_data _U_, void *private_data)
+void discoverylogout_cb(struct iscsi_context *iscsi, int status, void *command_data, void *private_data)
 {
 	struct client_state *state = (struct client_state *)private_data;
 	
@@ -290,7 +290,7 @@ void discovery_cb(struct iscsi_context *iscsi, int status, void *command_data, v
 }
 
 
-void discoverylogin_cb(struct iscsi_context *iscsi, int status, void *command_data _U_, void *private_data)
+void discoverylogin_cb(struct iscsi_context *iscsi, int status, void *command_data, void *private_data)
 {
 	if (status != 0) {
 		fprintf(stderr, "Login failed. %s\n", iscsi_get_error(iscsi));
@@ -303,7 +303,7 @@ void discoverylogin_cb(struct iscsi_context *iscsi, int status, void *command_da
 	}
 }
 
-void discoveryconnect_cb(struct iscsi_context *iscsi, int status, void *command_data _U_, void *private_data)
+void discoveryconnect_cb(struct iscsi_context *iscsi, int status, void *command_data, void *private_data)
 {
 	if (status != 0) {
 		fprintf(stderr, "discoveryconnect_cb: connection failed : %s\n", iscsi_get_error(iscsi));
@@ -329,7 +329,7 @@ void print_help(void)
 	fprintf(stderr, "  -i, --initiator-name=iqn-name     Initiatorname to use\n");
 	fprintf(stderr, "  -d, --debug                       Print debug information\n");
 	fprintf(stderr, "  -s, --show-luns                   Show the luns for each target\n");
-	fprintf(stderr, "      --url                         Output targets in URL format\n");
+	fprintf(stderr, "  -U, --url                         Output targets in URL format\n");
 	fprintf(stderr, "                                    (does not work with -s)\n");
 	fprintf(stderr, "\n");
 	fprintf(stderr, "Help options:\n");
@@ -349,8 +349,9 @@ int main(int argc, char *argv[])
 	struct iscsi_context *iscsi;
 	struct iscsi_url *iscsi_url = NULL;
 	struct client_state state;
-	const char *url = NULL;
-	int i;
+	char *url = NULL;
+	int c;
+	int option_index;
 	static int show_help = 0, show_usage = 0, debug = 0;
 
 #ifdef _WIN32
@@ -360,30 +361,44 @@ int main(int argc, char *argv[])
 	}
 #endif
 
-        for (i = 1; i < argc; i++) {
-                if (!strcmp(argv[i], "-?") ||
-                    !strcmp(argv[i], "-h") ||
-                    !strcmp(argv[i], "--help")) {
-                                show_help = 1;
-                } else if (!strcmp(argv[i], "-u") ||
-                           !strcmp(argv[i], "-usage")) {
+	static struct option long_options[] = {
+		{"help",           no_argument,          NULL,        'h'},
+		{"usage",          no_argument,          NULL,        'u'},
+		{"debug",          no_argument,          NULL,        'd'},
+		{"initiator-name", required_argument,    NULL,        'i'},
+		{"show-luns",      no_argument,          NULL,        's'},
+		{"url",            no_argument,          NULL,        'U'},
+		{0, 0, 0, 0}
+	};
+
+	while ((c = getopt_long(argc, argv, "h?udi:sU", long_options,
+				&option_index)) != -1) {
+		switch (c) {
+		case 'h':
+		case '?':
+			show_help = 1;
+			break;
+		case 'u':
 			show_usage = 1;
-                } else if (!strcmp(argv[i], "-d") ||
-                           !strcmp(argv[i], "--debug")) {
+			break;
+		case 'd':
 			debug = 1;
-                } else if (!strcmp(argv[i], "-i") ||
-                           !strcmp(argv[i], "--initiator-name")) {
-			initiator = argv[++i];
-                } else if (!strcmp(argv[i], "-s") ||
-                           !strcmp(argv[i], "--show-luns")) {
+			break;
+		case 'i':
+			initiator = optarg;
+			break;
+		case 's':
 			showluns = 1;
-                } else if (!strcmp(argv[i], "-U") ||
-                           !strcmp(argv[i], "--url")) {
+			break;
+		case 'U':
 			useurls = 1;
-                } else if (!strncmp("iscsi://", argv[i], 8)) {
-                        url = strdup(argv[i]);
-                }
-        }
+			break;
+		default:
+			fprintf(stderr, "Unrecognized option '%c'\n\n", c);
+			print_help();
+			exit(0);
+		}
+	}
 
 	if (show_help != 0) {
 		print_help();
@@ -397,6 +412,9 @@ int main(int argc, char *argv[])
 
 	memset(&state, 0, sizeof(state));
 
+	if (argv[optind] != NULL) {
+		url = strdup(argv[optind]);
+	}
 	if (url == NULL) {
 		fprintf(stderr, "You must specify iscsi target portal.\n");
 		print_usage();
@@ -418,10 +436,8 @@ int main(int argc, char *argv[])
 	}
 
 	iscsi_url = iscsi_parse_portal_url(iscsi, url);
-	
-	if (url) {
-		free(discard_const(url));
-	}
+
+        free(url);
 
 	if (iscsi_url == NULL) {
 		fprintf(stderr, "Failed to parse URL: %s\n", 
