@@ -25,17 +25,18 @@ THE SOFTWARE.
 #ifndef win32_COMPAT_H_
 #define win32_COMPAT_H_
 
-#ifdef _WIN32
-#define NO_IPv6 1
-
 #include <winsock2.h>
 #include <ws2tcpip.h>
-#include <Ws2ipdef.h>
+#include <ws2ipdef.h>
 #include <basetsd.h>
 #include <io.h>
 #include <malloc.h>
 #include <sys/stat.h>
 
+#if defined(_MSC_VER)
+#define __attribute__(attr)
+#endif
+
 #define SOL_TCP IPPROTO_TCP
 
 #if(_WIN32_WINNT < 0x0600)
@@ -54,10 +55,12 @@ struct pollfd {
 };
 #endif
 
+#ifdef _MSC_VER
 typedef int ssize_t;
 typedef int uid_t;
 typedef int gid_t;
 typedef int socklen_t;
+#endif
 
 /* Wrapper macros to call misc. functions win32 is missing */
 #define close                closesocket
@@ -70,12 +73,21 @@ typedef int socklen_t;
 #define poll(x, y, z)        win32_poll(x, y, z)
 #define inet_pton(x,y,z)     win32_inet_pton(x,y,z)
 #define sleep(x)             Sleep(x * 1000)
+#ifdef _MSC_VER
 #define getpid               GetCurrentProcessId
+#endif
 
 #if defined(_MSC_VER) && _MSC_VER < 1900
 #define snprintf(a, b, c, ...) _snprintf_s(a, b, b, c, ## __VA_ARGS__)
 #endif
 
+struct timezone;
+
+struct iovec {
+    void *iov_base;
+    size_t iov_len;
+};
+
 int     win32_inet_pton(int af, const char * src, void * dst);
 int     win32_poll(struct pollfd *fds, unsigned int nfsd, int timeout);
 int     win32_gettimeofday(struct timeval *tv, struct timezone *tz);
@@ -83,12 +95,6 @@ ssize_t win32_writev(int fd, const struct iovec *iov, int iovcnt);
 ssize_t win32_readv(int fd, const struct iovec *iov, int iovcnt);
 int     win32_dup2(int oldfd, int newfd);
 
-struct iovec {
-    void *iov_base;
-    size_t iov_len;
-};
+#define inline __inline__
 
-#define inline
-
-#endif // _WIN32
 #endif // win32_COMPAT_H_
